@@ -9,9 +9,10 @@ const Token::Type Lexer::tokenTypes[] =
 	Token::Type(DEF,     "'def'",                   "def"),
 	Token::Type(ASSIGN,  "'='",                     "="),
 	Token::Type(COLON,   "':'",                     ":"),
+	Token::Type(WILDCARD,"'_'",                     "_"),
 	Token::Type(DOT,     "a dot",                   "\\."),
 	Token::Type(COMMA,   "a comma",                 ","),
-	Token::Type(ID,      "an identifier",           "([[:alpha:]][[:alnum:]]*)|([!#$%&*+-/:<=>?@\\^_\\|~]+)"),
+	Token::Type(ID,      "an identifier",           "([[:alpha:]][[:alnum:]]*)|([!#$%&*+/:<=>?@^_|~-]+)"),
 	Token::Type(STR,     "a string",                "([\"\'])(\\\\[\"\']|[^\"\'[:cntrl:]])*[\"\']"),
 	Token::Type(NUM,     "a number",                "[[:digit:]]+|(0x|0X)[[:xdigit:]]+"),
 	Token::Type(LPAR,    "a beginning of sequence", "\\("),
