@@ -5,7 +5,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -34,6 +34,8 @@
 #define BUILDING_DELAY 30
 #define AREAS_DELAY 50
 
+using namespace boost;
+
 void AIWarrush::init(Player *player)
 {
 	assert(player);
@@ -300,7 +302,7 @@ bool AIWarrush::percentageOfBuildingsAreFullyWorked(int percentage)const
 	return num_worked_buildings * 100 >= num_buildings * percentage;
 }
 
-Order *AIWarrush::getOrder(void)
+boost::shared_ptr<Order> AIWarrush::getOrder(void)
 {
 	// reduce delays
 	if (buildingDelay > 0)
@@ -401,27 +403,28 @@ Order *AIWarrush::getOrder(void)
 		if(out_of_date_swarm)
 		{
 			Sint32 settings[3] = {4,1,3};
-			return new OrderModifySwarm(out_of_date_swarm->gid, settings);
+			return shared_ptr<Order>(new OrderModifySwarm(out_of_date_swarm->gid, settings));
 		}
 	}
 	
 	//all swarms should always have 5 workers at them!
 	Building *weak_swarm = getBuildingWithoutWorkersAssigned(IntBuildingType::SWARM_BUILDING, 5);
-	if (weak_swarm) return new OrderModifyBuilding(weak_swarm->gid, 5);
+	if (weak_swarm) return shared_ptr<Order>(new OrderModifyBuilding(weak_swarm->gid, 5));
 
 	//all inns should always have 3 workers at them! (best to build fast, make sure they're fed)
 	Building *weak_inn = getBuildingWithoutWorkersAssigned(IntBuildingType::FOOD_BUILDING, 3);
-	if (weak_inn) return new OrderModifyBuilding(weak_inn->gid, 3);
+	if (weak_inn) return shared_ptr<Order>(new OrderModifyBuilding(weak_inn->gid, 3));
 	
 	//work barracks more too.
 	Building *weak_barracks = getBuildingWithoutWorkersAssigned(IntBuildingType::ATTACK_BUILDING, 3);
-	if (weak_barracks && weak_barracks->constructionResultState != Building::NO_CONSTRUCTION) return new OrderModifyBuilding(weak_barracks->gid, 3);
+	if (weak_barracks && weak_barracks->constructionResultState != Building::NO_CONSTRUCTION)
+	 return shared_ptr<Order>(new OrderModifyBuilding(weak_barracks->gid, 3));
 	
 	//nothing at all to do?!
-	return new NullOrder;
+	return shared_ptr<Order>(new NullOrder);
 }
 
-Order *AIWarrush::pruneGuardAreas()
+boost::shared_ptr<Order> AIWarrush::pruneGuardAreas()
 {
 	//If we have any guard areas that aren't adjacent to an enemy building, we remove them.
 	BrushAccumulator acc;
@@ -450,19 +453,19 @@ Order *AIWarrush::pruneGuardAreas()
 				}
 				if(!keep)
 				{
-					acc.applyBrush(map,BrushApplication(x,y,0));
+					acc.applyBrush(BrushApplication(x,y,0), map);
 				}
 			}
 		}
 	}
 	if(acc.getApplicationCount())
 	{
-		return new OrderAlterateGuardArea(team->teamNumber,BrushTool::MODE_DEL,&acc);
+		return shared_ptr<Order>(new OrderAlterateGuardArea(team->teamNumber,BrushTool::MODE_DEL,&acc,map));
 	}
-	else return new NullOrder;
+	else return shared_ptr<Order>(new NullOrder);
 }
 	
-Order *AIWarrush::placeGuardAreas()
+boost::shared_ptr<Order> AIWarrush::placeGuardAreas()
 {
 	BrushAccumulator guard_add_acc;
 	//Place guard area on an enemy building if there is one...
@@ -498,7 +501,7 @@ Order *AIWarrush::placeGuardAreas()
 							{
 								for(int y = 0; y < bt->height; y++)
 								{
-									guard_add_acc.applyBrush(map,BrushApplication(b->posX+x,b->posY+y,6));
+									guard_add_acc.applyBrush(BrushApplication(b->posX+x,b->posY+y,6), map);
 								}
 							}
 						}
@@ -510,12 +513,12 @@ Order *AIWarrush::placeGuardAreas()
 	
 	if(guard_add_acc.getApplicationCount())
 	{
-		return new OrderAlterateGuardArea(team->teamNumber,BrushTool::MODE_ADD,&guard_add_acc);
+		return shared_ptr<Order>(new OrderAlterateGuardArea(team->teamNumber,BrushTool::MODE_ADD,&guard_add_acc, map));
 	}
-	else return new NullOrder;
+	else return shared_ptr<Order>(new NullOrder);
 }
 	
-Order *AIWarrush::farm()
+boost::shared_ptr<Order> AIWarrush::farm()
 {
 	// Algorithm initially stolen from Nicowar.
 	DynamicGradientMapArray water_gradient(map->w,map->h);
@@ -559,14 +562,14 @@ Order *AIWarrush::farm()
 						&& !map->isRessourceTakeable(x, y, PRUNE)
 						)
 					{
-						del_acc.applyBrush(map, BrushApplication(x, y, 0));
+						del_acc.applyBrush(BrushApplication(x, y, 0), map);
 					}
 				}
 			}
 			
 			if(map->isForbidden(x, y, team->me) && map->isClearArea(x, y, team->me))
 			{
-				del_acc.applyBrush(map, BrushApplication(x, y, 0));
+				del_acc.applyBrush(BrushApplication(x, y, 0), map);
 			}
 			
 			//we never clear anything but wood
@@ -574,7 +577,7 @@ Order *AIWarrush::farm()
 			{
 				if(map->isClearArea(x, y, team->me))
 				{
-					clr_del_acc.applyBrush(map, BrushApplication(x, y, 0));
+					clr_del_acc.applyBrush(BrushApplication(x, y, 0), map);
 				}
 			}
 
@@ -591,7 +594,7 @@ Order *AIWarrush::farm()
 									|| (map->getBuilding(x+xmod,y+ymod)!=NOGBID
 									&& (team->me & game->teams[Building::GIDtoTeam(map->getBuilding(x+xmod,y+ymod))]->me)))
 							{
-								clr_add_acc.applyBrush(map, BrushApplication(x, y, 0));
+								clr_add_acc.applyBrush(BrushApplication(x, y, 0), map);
 								goto doublebreak;
 							}
 						}
@@ -608,7 +611,7 @@ Order *AIWarrush::farm()
 				{
 					if(!map->isForbidden(x, y, team->me) && !map->isClearArea(x, y, team->me) && map->isMapDiscovered(x, y, team->me) && water_gradient(x, y) > (255 - 15))
 					{	
-						add_acc.applyBrush(map, BrushApplication(x, y, 0));
+						add_acc.applyBrush(BrushApplication(x, y, 0), map);
 					}
 				}
 			}
@@ -618,8 +621,8 @@ Order *AIWarrush::farm()
 				if(map->isRessourceTakeable(x, y, CORN))
 				{
 					if(!map->isForbidden(x, y, team->me) && map->isMapDiscovered(x, y, team->me) && water_gradient(x, y) > (255 - 15))
-					{	
-						add_acc.applyBrush(map, BrushApplication(x, y, 0));
+					{
+						add_acc.applyBrush(BrushApplication(x, y, 0), map);
 					}
 				}
 			}
@@ -633,27 +636,27 @@ Order *AIWarrush::farm()
 				&& map->isMapDiscovered(x, y, team->me)
 					)
 			{
-				add_acc.applyBrush(map, BrushApplication(x, y, 0));
+				add_acc.applyBrush(BrushApplication(x, y, 0), map);
 			}
 
 		}
 	}
 
 	if(del_acc.getApplicationCount()>0)
-		return new OrderAlterateForbidden(team->teamNumber, BrushTool::MODE_DEL, &del_acc);
+		return shared_ptr<Order>(new OrderAlterateForbidden(team->teamNumber, BrushTool::MODE_DEL, &del_acc, map));
 	if(add_acc.getApplicationCount()>0)
-		return new OrderAlterateForbidden(team->teamNumber, BrushTool::MODE_ADD, &add_acc);
+		return shared_ptr<Order>(new OrderAlterateForbidden(team->teamNumber, BrushTool::MODE_ADD, &add_acc, map));
 	if(clr_del_acc.getApplicationCount()>0)
-		return new OrderAlterateClearArea(team->teamNumber, BrushTool::MODE_DEL, &clr_del_acc);
+		return shared_ptr<Order>(new OrderAlterateClearArea(team->teamNumber, BrushTool::MODE_DEL, &clr_del_acc, map));
 	if(clr_add_acc.getApplicationCount()>0)
-		return new OrderAlterateClearArea(team->teamNumber, BrushTool::MODE_ADD, &clr_add_acc);
+		return shared_ptr<Order>(new OrderAlterateClearArea(team->teamNumber, BrushTool::MODE_ADD, &clr_add_acc, map));
 
 	//nothing to do...
-	return new NullOrder();
+	return shared_ptr<Order>(new NullOrder());
 }
 
 //Simple hack to place explore flags on opponents' starting swarms.
-Order *AIWarrush::setupExploreFlagForTeam(Team *enemy_team)
+boost::shared_ptr<Order> AIWarrush::setupExploreFlagForTeam(Team *enemy_team)
 {
 	if(verbose)std::cout << "looking for swarms:\n";
 	for(int j=0;j<1024;j++)
@@ -662,7 +665,7 @@ Order *AIWarrush::setupExploreFlagForTeam(Team *enemy_team)
 		if((b)&&(b->type->shortTypeNum == IntBuildingType::SWARM_BUILDING)&&(b->constructionResultState == Building::NO_CONSTRUCTION))
 		{
 			Sint32 typeNum=globalContainer->buildingsTypes.getTypeNum("explorationflag", 0, false);
-			return new OrderCreate(team->teamNumber, b->posX, b->posY, typeNum, 1, 1);
+			return shared_ptr<Order>(new OrderCreate(team->teamNumber, b->posX, b->posY, typeNum, 1, 1));
 		}
 	}
 	if(verbose)std::cout << "No swarms found\n";
@@ -673,7 +676,7 @@ Order *AIWarrush::setupExploreFlagForTeam(Team *enemy_team)
 		if(b)
 		{
 			Sint32 typeNum=globalContainer->buildingsTypes.getTypeNum("explorationflag", 0, false);
-			return new OrderCreate(team->teamNumber, b->posX, b->posY, typeNum, 1, 1);
+			return shared_ptr<Order>(new OrderCreate(team->teamNumber, b->posX, b->posY, typeNum, 1, 1));
 		}
 	}
 	if(verbose)std::cout << "No buildings found\n";
@@ -684,12 +687,12 @@ Order *AIWarrush::setupExploreFlagForTeam(Team *enemy_team)
 		if(u)
 		{
 			Sint32 typeNum=globalContainer->buildingsTypes.getTypeNum("explorationflag", 0, false);
-			return new OrderCreate(team->teamNumber, u->posX, u->posY, typeNum, 1, 1);
+			return shared_ptr<Order>(new OrderCreate(team->teamNumber, u->posX, u->posY, typeNum, 1, 1));
 		}
 	}
 	//what, enemy has no buildings or units at the beginning of the game? o_O O_o o_O
 	if(verbose)std::cout << "No buildings found, no units found o_O\n";
-	return new NullOrder;
+	return shared_ptr<Order>(new NullOrder);
 }
 
 bool AIWarrush::locationIsAvailableForBuilding(int x, int y, int width, int height)
@@ -752,7 +755,7 @@ void AIWarrush::initializeGradientWithResource(DynamicGradientMapArray &gradient
 	}
 }
 
-Order *AIWarrush::buildBuildingOfType(Sint32 shortTypeNum)
+boost::shared_ptr<Order> AIWarrush::buildBuildingOfType(Sint32 shortTypeNum)
 {
 	
 	// set delay
@@ -801,7 +804,7 @@ Order *AIWarrush::buildBuildingOfType(Sint32 shortTypeNum)
 	if (!swarm)
 	{
 		if(verbose)std::cout << "No swarm found!\n";
-		return new NullOrder;
+		return shared_ptr<Order>(new NullOrder);
 	}
 	Sint32 destination_x,destination_y;
 	{
@@ -830,6 +833,6 @@ Order *AIWarrush::buildBuildingOfType(Sint32 shortTypeNum)
 		
 	// create and return order
 	Sint32 typeNum=globalContainer->buildingsTypes.getTypeNum(IntBuildingType::typeFromShortNumber(shortTypeNum), 0, true);
-	return new OrderCreate(team->teamNumber, destination_x, destination_y, typeNum, 1, 1);
+	return shared_ptr<Order>(new OrderCreate(team->teamNumber, destination_x, destination_y, typeNum, 1, 1));
 }
 
