@@ -76,6 +76,7 @@ Building::Building(int x, int y, Uint16 gid, Sint32 typeNum, Team *team, Buildin
 	maxUnitWorkingLocal = maxUnitWorking;
 	maxUnitWorkingPreferred = maxUnitWorking;
 	maxUnitWorkingFuture = unitWorkingFuture;
+	maxUnitWorkingPrevious = 0;
 	desiredMaxUnitWorking = maxUnitWorking;
 	subscriptionWorkingTimer = 0;
 
@@ -85,6 +86,8 @@ Building::Building(int x, int y, Uint16 gid, Sint32 typeNum, Team *team, Buildin
 	posXLocal=posX;
 	posYLocal=posY;
 
+	underAttackTimer = 0;
+
 	// flag usefull :
 	unitStayRange=type->defaultUnitStayRange;
 	unitStayRangeLocal=unitStayRange;
@@ -150,6 +153,10 @@ Building::Building(int x, int y, Uint16 gid, Sint32 typeNum, Team *team, Buildin
 	}
 	
 	verbose=false;
+	
+	lastShootStep = 0xFFFFFFFF;
+	lastShootSpeedX = 0;
+	lastShootSpeedY = 0;
 }
 
 Building::~Building()
@@ -198,6 +205,11 @@ void Building::load(GAGCore::InputStream *stream, BuildingsTypes *types, Team *o
 	posXLocal = posX;
 	posYLocal = posY;
 
+	if(versionMinor>=61)
+		underAttackTimer = stream->readUint8("underAttackTimer");
+	else
+		underAttackTimer = 0;
+
 	// Flag specific
 	unitStayRange = stream->readUint32("unitStayRange");
 	unitStayRangeLocal = unitStayRange;
@@ -286,6 +298,10 @@ void Building::load(GAGCore::InputStream *stream, BuildingsTypes *types, Team *o
 	
 	verbose = false;
 	stream->readLeaveSection();
+	
+	lastShootStep = 0xFFFFFFFF;
+	lastShootSpeedX = 0;
+	lastShootSpeedY = 0;
 }
 
 void Building::save(GAGCore::OutputStream *stream)
@@ -304,6 +320,8 @@ void Building::save(GAGCore::OutputStream *stream)
 	stream->writeSint32(posX, "posX");
 	stream->writeSint32(posY, "posY");
 
+	stream->writeUint8(underAttackTimer, "underAttackTimer");
+
 	// Flag specific
 	stream->writeUint32(unitStayRange, "unitStayRange");
 	for(int i=0; i<BASIC_COUNT; i++)
@@ -382,6 +400,12 @@ void Building::loadCrossRef(GAGCore::InputStream *stream, BuildingsTypes *types,
 	subscriptionWorkingTimer = stream->readSint32("subscriptionWorkingTimer");
 	maxUnitWorking = stream->readSint32("maxUnitWorking");
 	maxUnitWorkingPreferred = stream->readSint32("maxUnitWorkingPreferred");
+	if(versionMinor>=65)
+		maxUnitWorkingPrevious = stream->readSint32("maxUnitWorkingPrevious");
+	else
+		maxUnitWorkingPrevious = maxUnitWorkingPreferred;
+	if(versionMinor>=70)
+		maxUnitWorkingFuture = stream->readSint32("maxUnitWorkingFuture");
 	maxUnitWorkingLocal = maxUnitWorking;
 	desiredMaxUnitWorking = maxUnitWorking;
 	
@@ -409,6 +433,10 @@ void Building::saveCrossRef(GAGCore::OutputStream *stream)
 	
 	// units
 	stream->writeSint32(maxUnitInside, "maxUnitInside");
+	//TODO: std::list::size() is O(n). We should investigate
+	//if our intense use of this has an impact on overall performance.
+	//steph and nuage suggested to store and update size in a variable
+	//what is faster but also more error prone.
 	stream->writeUint32(unitsWorking.size(), "nbWorking");
 	fprintf(logFile, " nbWorking=%zd\n", unitsWorking.size());
 	i = 0;
@@ -424,6 +452,8 @@ void Building::saveCrossRef(GAGCore::OutputStream *stream)
 	stream->writeSint32(subscriptionWorkingTimer, "subscriptionWorkingTimer");
 	stream->writeSint32(maxUnitWorking, "maxUnitWorking");
 	stream->writeSint32(maxUnitWorkingPreferred, "maxUnitWorkingPreferred");
+	stream->writeSint32(maxUnitWorkingPrevious, "maxUnitWorkingPrevious");
+	stream->writeSint32(maxUnitWorkingFuture, "maxUnitWorkingFuture");
 	
 	stream->writeUint32(unitsInside.size(), "nbInside");
 	fprintf(logFile, " nbInside=%zd\n", unitsInside.size());
@@ -482,7 +512,7 @@ void Building::wishedRessources(int needs[MAX_NB_RESSOURCES])
 {
 	 // we balance the system with Units working on it:
 	for (int ri = 0; ri < MAX_NB_RESSOURCES; ri++)
-		needs[ri] = (2 * (type->maxRessource[ri] - ressources[ri])) / (type->multiplierRessource[ri]);
+		needs[ri] = (4 * (type->maxRessource[ri] - ressources[ri])) / (type->multiplierRessource[ri] * 3);
 	for (std::list<Unit *>::iterator ui = unitsWorking.begin(); ui != unitsWorking.end(); ++ui)
 		if ((*ui)->destinationPurprose >= 0)
 		{
@@ -495,7 +525,7 @@ void Building::computeWishedRessources()
 {
 	 // we balance the system with Units working on it:
 	for (int ri = 0; ri < MAX_NB_RESSOURCES; ri++)
-		wishedResources[ri] = (2 * (type->maxRessource[ri] - ressources[ri])) / (type->multiplierRessource[ri]);
+		wishedResources[ri] = (4 * (type->maxRessource[ri] - ressources[ri])) / (type->multiplierRessource[ri] * 3);
 	for (std::list<Unit *>::iterator ui = unitsWorking.begin(); ui != unitsWorking.end(); ++ui)
 		if ((*ui)->destinationPurprose >= 0)
 		{
@@ -552,12 +582,13 @@ void Building::launchConstruction(Sint32 unitWorking, Sint32 unitWorkingFuture)
 			Unit *u=*it;
 			assert(u);
 			int d=u->displacement;
-			if ((d!=Unit::DIS_INSIDE)&&(d!=Unit::DIS_ENTERING_BUILDING))
+			if ((d!=Unit::DIS_INSIDE)&&(d!=Unit::DIS_ENTERING_BUILDING)&&(d!=Unit::DIS_EXITING_BUILDING))
 			{
 				u->standardRandomActivity();
 				unitsToRemove.push_front(u);
 			}
 		}
+		
 		for (std::list<Unit *>::iterator it=unitsToRemove.begin(); it!=unitsToRemove.end(); ++it)
 		{
 			Unit *u=*it;
@@ -565,6 +596,7 @@ void Building::launchConstruction(Sint32 unitWorking, Sint32 unitWorkingFuture)
 			unitsInside.remove(u);
 		}
 		
+		maxUnitWorkingPrevious = maxUnitWorking;
 		buildingState=WAITING_FOR_CONSTRUCTION;
 		maxUnitWorkingLocal=0;
 		maxUnitWorking=0;
@@ -608,6 +640,9 @@ void Building::cancelConstruction(Sint32 unitWorking)
 	}
 	else if (buildingState==WAITING_FOR_CONSTRUCTION_ROOM)
 	{
+		if(constructionResultState == UPGRADE)
+			removeForbiddenZoneFromUpgradeArea();
+			
 		owner->buildingsTryToBuildingSiteRoom.remove(this);
 		buildingState=ALIVE;
 	}
@@ -621,8 +656,6 @@ void Building::cancelConstruction(Sint32 unitWorking)
 		// when the building upgrade" was already canceled.
 		return;
 	}
-	
-	removeForbiddenZoneFromUpgradeArea();
 
 	constructionResultState=NO_CONSTRUCTION;
 	
@@ -647,12 +680,9 @@ void Building::cancelConstruction(Sint32 unitWorking)
 
 	if (!type->isVirtual)
 		owner->map->setBuilding(posX, posY, type->width, type->height, gid);
-	
-	if (type->maxUnitWorking)
-		maxUnitWorking=maxUnitWorkingPreferred;
-	else
-		maxUnitWorking=0;
-	maxUnitWorkingLocal=unitWorking; //maxUnitWorking;
+
+	maxUnitWorking=maxUnitWorkingPrevious;
+	maxUnitWorkingLocal=maxUnitWorking; //maxUnitWorking;
 	maxUnitInside=type->maxUnitInside;
 	updateCallLists();
 	updateUnitsWorking();
@@ -690,6 +720,7 @@ void Building::launchDelete(void)
 	if (buildingState==ALIVE)
 	{
 		buildingState=WAITING_FOR_DESTRUCTION;
+		maxUnitWorkingPrevious = maxUnitWorking;
 		maxUnitWorking=0;
 		maxUnitWorkingLocal=0;
 		maxUnitInside=0;
@@ -703,10 +734,7 @@ void Building::launchDelete(void)
 void Building::cancelDelete(void)
 {
 	buildingState=ALIVE;
-	if (type->maxUnitWorking)
-		maxUnitWorking=maxUnitWorkingPreferred;
-	else
-		maxUnitWorking=0;
+	maxUnitWorking=maxUnitWorkingPrevious;
 	maxUnitWorkingLocal=maxUnitWorking;
 	maxUnitInside=type->maxUnitInside;
 	updateCallLists();
@@ -832,7 +860,8 @@ void Building::updateConstructionState(void)
 			{
 				buildingState=WAITING_FOR_CONSTRUCTION_ROOM;
 				owner->buildingsTryToBuildingSiteRoom.push_front(this);
-				addForbiddenZoneToUpgradeArea();
+				if(constructionResultState == UPGRADE)
+					addForbiddenZoneToUpgradeArea();
 				if (verbose)
 					printf("bgid=%d, inserted in buildingsTryToBuildingSiteRoom\n", gid);
 			}
@@ -852,8 +881,6 @@ void Building::updateBuildingSite(void)
 		for(int i=0; i<MAX_RESSOURCES; i++)
 			ressources[i]-=type->maxRessource[i];
 
-		if(constructionResultState!=NEW_BUILDING)
-			removeForbiddenZoneFromUpgradeArea();
 		owner->prestige-=type->prestige;
 		typeNum=type->nextLevel;
 		type=globalContainer->buildingsTypes.get(type->nextLevel);
@@ -1064,6 +1091,9 @@ bool Building::tryToBuildingSiteRoom(void)
 	bool isRoom=owner->map->isFreeForBuilding(newPosX, newPosY, newWidth, newHeight, gid);
 	if (isRoom)
 	{
+		if(constructionResultState == UPGRADE)
+			removeForbiddenZoneFromUpgradeArea();
+		
 		// OK, we have found enough room to expand our building-site, then we set-up the building-site.
 		if (constructionResultState==REPAIR)
 		{
@@ -1161,25 +1191,24 @@ void Building::addForbiddenZoneToUpgradeArea(void)
 	int midPosY=posY-type->decTop;
 
 	int targetLevelTypeNum=-1;
-	if (constructionResultState==UPGRADE)
-		targetLevelTypeNum=type->nextLevel;
-	else if (constructionResultState==REPAIR)
-		targetLevelTypeNum=type->prevLevel;
-	else
-		assert(false);
+	targetLevelTypeNum=type->nextLevel;
 
 	BuildingType *targetBt=globalContainer->buildingsTypes.get(targetLevelTypeNum);
 	int newPosX=midPosX+targetBt->decLeft;
 	int newPosY=midPosY+targetBt->decTop;
 	int newWidth=targetBt->width;
 	int newHeight=targetBt->height;
-	for(int nx=0; nx<newWidth; ++nx)
+	
+	for(int x=newPosX; x<(newPosX+newWidth); ++x)
 	{
-		for(int ny=0; ny<newHeight; ++ny)
+		for(int y=newPosY; y<(newPosY+newHeight); ++y)
 		{
-			owner->map->addHiddenForbidden(owner->map->normalizeX(newPosX+nx), owner->map->normalizeY(newPosY+ny), owner->teamNumber); 
+			owner->map->addForbidden(x, y, owner->teamNumber);
 		}
 	}
+	if(owner == owner->game->gui->getLocalTeam())
+		owner->map->computeLocalForbidden(owner->teamNumber);
+	owner->map->updateForbiddenGradient(owner->teamNumber);
 }
 
 
@@ -1190,25 +1219,24 @@ void Building::removeForbiddenZoneFromUpgradeArea(void)
 	int midPosY=posY-type->decTop;
 
 	int targetLevelTypeNum=-1;
-	if (constructionResultState==UPGRADE)
-		targetLevelTypeNum=type->level;
-	else if (constructionResultState==REPAIR)
-		targetLevelTypeNum=type->level;
-	else
-		assert(false);
+	targetLevelTypeNum=type->nextLevel;
 
 	BuildingType *targetBt=globalContainer->buildingsTypes.get(targetLevelTypeNum);
 	int newPosX=midPosX+targetBt->decLeft;
 	int newPosY=midPosY+targetBt->decTop;
 	int newWidth=targetBt->width;
 	int newHeight=targetBt->height;
-	for(int nx=0; nx<newWidth; ++nx)
+	
+	for(int x=newPosX; x<(newPosX+newWidth); ++x)
 	{
-		for(int ny=0; ny<newHeight; ++ny)
+		for(int y=newPosY; y<(newPosY+newHeight); ++y)
 		{
-			owner->map->removeHiddenForbidden(owner->map->normalizeX(newPosX+nx), owner->map->normalizeY(newPosY+ny), owner->teamNumber); 
+			owner->map->removeForbidden(x, y, owner->teamNumber);
 		}
 	}
+	if(owner == owner->game->gui->getLocalTeam())
+		owner->map->computeLocalForbidden(owner->teamNumber);
+	owner->map->updateForbiddenGradient(owner->teamNumber);
 }
 
 
@@ -1268,14 +1296,17 @@ int Building::desiredNumberOfWorkers(void)
 			neededRessourcesSum += neededRessources;
 	}
 	int user_num = maxUnitWorking;
-	int max_considering_ressources = 2 * neededRessourcesSum;
+	int max_considering_ressources = (4 * neededRessourcesSum) / 3;
 	return std::min(user_num, max_considering_ressources);
 }
 
 
 void Building::step(void)
 {
-	desiredMaxUnitWorking = desiredNumberOfWorkers();
+	updateCallLists();
+	if(underAttackTimer > 0)
+		underAttackTimer-=1;
+
 	// NOTE : Unit needs to update itself when it is in a building
 }
 
@@ -1288,7 +1319,6 @@ void Building::subscribeToBringRessourcesStep()
 		printf("bgid=%d, subscribeToBringRessourcesStep()...\n", gid);
 	if (((Sint32)unitsWorking.size()<desiredMaxUnitWorking) /* && !unitsWorkingSubscribe.empty() */ )
 	{
-		int maxValue=-INT_MAX;
 		Unit *choosen=NULL;
 		Map *map=owner->map;
 		/* To choose a good unit, we get a composition of things:
@@ -1300,6 +1330,8 @@ void Building::subscribeToBringRessourcesStep()
 
 		score_to_max=(rightRes*100/d+noRes*80/(d+dr)+wrongRes*25/(d+dr))/walk+sign(timeleft>>2 - (d+dr))*500+100/harvest
 		*/
+		/*
+		int maxValue=-INT_MAX;
 		for(int n=0; n<1024; ++n)
 		{
 			Unit* unit=owner->myUnits[n];
@@ -1359,7 +1391,9 @@ void Building::subscribeToBringRessourcesStep()
 				choosen=unit;
 			}
 		}
-/*
+*/
+		int maxLevel = -1;
+		int minValue = INT_MAX;
 		//First: we look only for units with a needed resource:
 		for(int n=0; n<1024; ++n)
 		{
@@ -1489,7 +1523,6 @@ void Building::subscribeToBringRessourcesStep()
 				}
 			}
 		}
-*/
 		if (choosen)
 		{
 			if (verbose)
@@ -1561,7 +1594,7 @@ void Building::subscribeForFlagingStep()
 				for(int n=0; n<1024; ++n)
 				{
 					Unit* unit=owner->myUnits[n];
-					if(unit==NULL || unit->activity != Unit::ACT_RANDOM || unit->medical != Unit::MED_FREE)
+					if(unit==NULL || unit->activity != Unit::ACT_RANDOM || unit->medical != Unit::MED_FREE || unit->movement == Unit::MOV_ATTACKING_TARGET)
 						continue;
 			  	 	if(!canUnitWorkHere(unit))
 			  		  	continue;
@@ -1569,7 +1602,7 @@ void Building::subscribeForFlagingStep()
 					int timeLeft=unit->hungry/unit->race->hungryness;
 					int hp=(unit->hp<<4)/unit->race->unitTypes[0][0].performance[HP];
 					int dist;
-					if (map->buildingAvailable(this, unit->performance[SWIM]>0, unit->posX, unit->posY, &dist) && (dist<timeLeft))
+					if (map->buildingAvailable(this, (unit->performance[SWIM]>0 ? true : false), unit->posX, unit->posY, &dist) && (dist<timeLeft))
 					{
 						int value=dist-2*timeLeft-2*hp;
 						//We want to maximize the attack level, use higher level soldeirs first
@@ -1718,7 +1751,7 @@ void Building::swarmStep(void)
 }
 
 
-void Building::turretStep(void)
+void Building::turretStep(Uint32 stepCounter)
 {
 	// create bullet from stones in stock
 	if (ressources[STONE]>0 && (bullets<=(type->maxBullets-type->multiplierStoneToBullets)))
@@ -2006,6 +2039,9 @@ void Building::turretStep(void)
 			s->bullets.push_front(b);
 			bullets--;
 			shootingCooldown = SHOOTING_COOLDOWN_MAX;
+			lastShootStep = stepCounter;
+			lastShootSpeedX = speedX;
+			lastShootSpeedY = speedY;
 		}
 	}
 
@@ -2079,6 +2115,7 @@ void Building::kill(void)
 		owner->dirtyGlobalGradient();
 		owner->map->updateForbiddenGradient(owner->teamNumber);
 		owner->map->updateGuardAreasGradient(owner->teamNumber);
+		owner->map->updateClearAreasGradient(owner->teamNumber);
 		if (type->isBuildingSite && type->level==0)
 		{
 			bool good=false;
@@ -2498,64 +2535,80 @@ Uint32 Building::checkSum(std::vector<Uint32> *checkSumsVector)
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [0]
 		
-
 	cs^=buildingState;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [1]
 	cs=(cs<<31)|(cs>>1);
+		
+	cs^=constructionResultState;
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [2]
+	cs=(cs<<31)|(cs>>1);
 	
 	cs^=maxUnitWorking;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [2]
+		checkSumsVector->push_back(cs);// [3]
+	
+	cs^=maxUnitWorkingFuture;
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [4]
 	
 	cs^=maxUnitWorkingPreferred;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [3]
+		checkSumsVector->push_back(cs);// [5]
+	
+	cs^=maxUnitWorkingPrevious;
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [7]
+	
+	cs^=desiredMaxUnitWorking;
+	if (checkSumsVector)
+		checkSumsVector->push_back(cs);// [8]
 		
 	cs^=unitsWorking.size();
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [4]
+		checkSumsVector->push_back(cs);// [9]
 
-	cs^=maxUnitInside;
+	cs^=subscriptionWorkingTimer;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [5]
-
+		checkSumsVector->push_back(cs);// [10]
+	
 	cs^=unitsInside.size();
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [6]
+		checkSumsVector->push_back(cs);// [11]
 	cs=(cs<<31)|(cs>>1);
 
 	cs^=posX;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [7]
+		checkSumsVector->push_back(cs);// [12]
 	
 	cs^=posY;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [8]
+		checkSumsVector->push_back(cs);// [13]
 	cs=(cs<<31)|(cs>>1);
 
 	cs^=unitStayRange;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [9]
+		checkSumsVector->push_back(cs);// [14]
 
 	for (int i=0; i<MAX_RESSOURCES; i++)
 		cs^=localRessource[i];
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [10]
+		checkSumsVector->push_back(cs);// [15]
 	cs=(cs<<31)|(cs>>1);
 
 	cs^=hp;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [11]
+		checkSumsVector->push_back(cs);// [16]
 
 	cs^=productionTimeout;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [12]
+		checkSumsVector->push_back(cs);// [17]
 		
 
 	cs^=totalRatio;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [13]
+		checkSumsVector->push_back(cs);// [18]
 	
 	
 	for (int i=0; i<NB_UNIT_TYPE; i++)
@@ -2565,30 +2618,30 @@ Uint32 Building::checkSum(std::vector<Uint32> *checkSumsVector)
 		cs=(cs<<31)|(cs>>1);
 	}
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [14]
+		checkSumsVector->push_back(cs);// [19]
 
 	cs^=shootingStep;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [15]
+		checkSumsVector->push_back(cs);// [20]
 
 
 	cs^=shootingCooldown;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [16]
+		checkSumsVector->push_back(cs);// [21]
 	
 		
 	cs^=bullets;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [17]
+		checkSumsVector->push_back(cs);// [22]
 	cs=(cs<<31)|(cs>>1);
 
 	cs^=seenByMask;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [18]
+		checkSumsVector->push_back(cs);// [23]
 	
 	cs^=gid;
 	if (checkSumsVector)
-		checkSumsVector->push_back(cs);// [19]
+		checkSumsVector->push_back(cs);// [24]
 	
 	return cs;
 }
