@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -37,7 +37,7 @@ CampaignScreen::CampaignScreen(const std::string &text)
 	addWidget(new TextButton(20, 20, 180, 40, ALIGN_RIGHT, ALIGN_BOTTOM, "menu", Toolkit::getStringTable()->getString("[Cancel]"), 1, 27));
 	
 	
-	addWidget(new TextArea(20, 20, 20, 80, ALIGN_FILL, ALIGN_FILL, "standard", true, text.c_str()));
+	addWidget(new TextArea(20, 20, 20, 80, ALIGN_FILL, ALIGN_FILL, "standard", true, Toolkit::getStringTable()->getString(text.c_str())));
 }
 
 void CampaignScreen::onAction(Widget *source, Action action, int par1, int par2)
