@@ -36,9 +36,11 @@
 #include "GlobalContainer.h"
 #include "LogFileManager.h"
 #include "Utilities.h"
-#include "YOGScreen.h"
+#include "YOGClientLobbyScreen.h"
 #include "SoundMixer.h"
 #include "CampaignScreen.h"
+#include "Player.h"
+#include "AIEcho.h"
 
 #include <iostream>
 
@@ -129,7 +131,7 @@ int Engine::initCustom(const std::string &gameName)
 		}
 	}
 
-	int ret = initGame(mapHeader, gameHeader, false);
+	int ret = initGame(mapHeader, gameHeader, true);
 	if(ret != EE_NO_ERROR)
 		return EE_CANT_LOAD_MAP;
 	else if(ret == -1)
@@ -178,6 +180,30 @@ int Engine::initMultiplayer(boost::shared_ptr<MultiplayerGame> multiplayerGame,
 
 
 
+void Engine::createRandomGame()
+{
+	MapHeader map = chooseRandomMap();
+	
+	std::cout<<"Randomly Chosen Map: "<<map.getMapName()<<std::endl;
+	
+	GameHeader game = createRandomGame(map.getNumberOfTeams());
+	std::cout<<"Random Seed gameheader: "<<game.getRandomSeed();
+	for (int p=0; p<game.getNumberOfPlayers(); p++)
+	{
+		std::cout<<"    Player: "<<game.getBasePlayer(p).name<<" for team "<<game.getBasePlayer(p).teamNumber<<std::endl;
+	}
+	
+	gui.localPlayer=0;
+	gui.localTeamNo=0;
+	
+	initGame(map, game);
+
+	// set the correct alliance
+	gui.game.setAIFFA();
+}
+
+
+
 bool Engine::haveMap(const MapHeader& mapHeader)
 {
 	InputStream *stream = new BinaryInputStream(Toolkit::getFileManager()->openInputStreamBackend(mapHeader.getFileName()));
@@ -208,7 +234,12 @@ int Engine::run(void)
 	if (globalContainer->runNoX)
 	{
 		printf("nox::game started\n");
-		noxStartTick = SDL_GetTicks();
+		automaticGameStartTick = SDL_GetTicks();
+	}
+	
+	if(!globalContainer->runNoX)
+	{
+		globalContainer->gfx->cursorManager.setDrawColor(gui.getLocalTeam()->color);
 	}
 	
 	while (doRunOnceAgain)
@@ -222,33 +253,37 @@ int Engine::run(void)
 		Sint32 startTime = SDL_GetTicks();
 		unsigned frameNumber = 0;
 
-		unsigned int skipOrders = 0;
-
 		while (gui.isRunning)
 		{
 			// We always allow the user to use the gui:
-			if (globalContainer->runNoX)
+			if (globalContainer->automaticEndingGame)
 			{
-				if (!gui.getLocalTeam()->isAlive)
+				if (!gui.getLocalTeam()->isAlive && !globalContainer->automaticGameGlobalEndConditions)
 				{
 					printf("nox::gui.localTeam is dead\n");
 					gui.isRunning = false;
-					noxEndTick = SDL_GetTicks();
+					automaticGameEndTick = SDL_GetTicks();
 				}
-				else if (gui.getLocalTeam()->hasWon)
+				else if (gui.getLocalTeam()->hasWon && !globalContainer->automaticGameGlobalEndConditions)
 				{
 					printf("nox::gui.localTeam has won\n");
 					gui.isRunning = false;
-					noxEndTick = SDL_GetTicks();
+					automaticGameEndTick = SDL_GetTicks();
 				}
 				else if (gui.game.totalPrestigeReached)
 				{
 					printf("nox::gui.game.totalPrestigeReached\n");
 					gui.isRunning = false;
-					noxEndTick = SDL_GetTicks();
+					automaticGameEndTick = SDL_GetTicks();
+				}
+				else if (gui.game.isGameEnded)
+				{
+					printf("nox::gui.game.isGameEnded\n");
+					gui.isRunning = false;
+					automaticGameEndTick = SDL_GetTicks();
 				}
 			}
-			else
+			if(!globalContainer->runNoX)
 				gui.step();
 	
 			if (!gui.hardPause)
@@ -298,7 +333,7 @@ int Engine::run(void)
 					if(!net->matchCheckSums())
 					{	
 						std::cout<<"Game desychronized."<<std::endl;
-						gui.game.dumpAllData("glob2.world-desynchronization.dump2.txt");
+						gui.game.dumpAllData("glob2.world-desynchronization.dump.txt");
 						assert(false);
 					}
 					else
@@ -318,16 +353,16 @@ int Engine::run(void)
 					gui.game.syncStep(gui.localTeamNo);
 			}
 
-			if (globalContainer->runNoX)
+			if (globalContainer->automaticEndingGame)
 			{
-				if ((int)gui.game.stepCounter == globalContainer->runNoXCountSteps)
+				if ((int)gui.game.stepCounter == globalContainer->automaticEndingSteps)
 				{
 					gui.isRunning = false;
-					noxEndTick = SDL_GetTicks();
+					automaticGameEndTick = SDL_GetTicks();
 					printf("nox::gui.game.checkSum() = %08x\n", gui.game.checkSum());
 				}
 			}
-			else
+			if(!globalContainer->runNoX)
 			{
 				// we draw
 				gui.drawAll(gui.localTeamNo);
@@ -380,15 +415,24 @@ int Engine::run(void)
 			}
 		}
 
+		if(globalContainer->automaticEndingGame)
+		{
+			int time = gui.game.stepCounter;
+			int seconds = (time / 25) % 60;
+			int minutes = (time / 25) / 60;
+			std::cout<< "automaticEndingGame ended: "<<time<<" ticks, "<<seconds<<" seconds, "<<minutes<<" minutes"<<std::endl;
+		}
+
 		cpuStats.format();
 
 		delete net;
 		net=NULL;
+		multiplayer.reset();
 		
 		if (gui.exitGlobCompletely)
 			return -1; // There is no bypass for the "close window button"
 
-	
+		
 		doRunOnceAgain=false;
 		
 		if (gui.toLoadGameFileName[0])
@@ -400,8 +444,13 @@ int Engine::run(void)
 		}
 	}
 	
-	if (globalContainer->runNoX)
+	if (globalContainer->runNoX || globalContainer->automaticEndingGame)
+	{
+
+		if(!globalContainer->runNoX)
+			globalContainer->gfx->cursorManager.setDefaultColor();
 		return -1;
+	}
 	else
 	{
 		// Restart menu music
@@ -412,6 +461,9 @@ int Engine::run(void)
 		EndGameScreen endGameScreen(&gui);
 		int result = endGameScreen.execute(globalContainer->gfx, 40);
 		
+		// Return to default color
+		globalContainer->gfx->cursorManager.setDefaultColor();
+		
 		// Return
 		return (result == -1) ? -1 : EE_NO_ERROR;
 	}
@@ -419,6 +471,68 @@ int Engine::run(void)
 
 
 
+MapHeader Engine::loadMapHeader(const std::string &filename)
+{
+	MapHeader mapHeader;
+	InputStream *stream = new BinaryInputStream(Toolkit::getFileManager()->openInputStreamBackend(filename));
+	if (stream->isEndOfStream())
+	{
+		std::cerr << "Engine::loadMapHeader : error, can't open file " << filename  << std::endl;
+	}
+	else
+	{
+		if (verbose)
+			std::cout << "Engine::loadMapHeader : loading map " << filename << std::endl;
+		bool validMapSelected = mapHeader.load(stream);
+		if (!validMapSelected)
+			std::cerr << "Engine::loadMapHeader : invalid map header for map " << filename << std::endl;
+	}
+	delete stream;
+	
+	//Map name is the filename without underscores or .map, it has to be updated in case the map file itself was renamed
+	std::string mapName;
+	if(mapHeader.getIsSavedGame())
+		mapName=filename.substr(filename.find("/")+1, filename.size()-6-filename.find("/"));
+	else
+		mapName=filename.substr(filename.find("/")+1, filename.size()-5-filename.find("/"));
+	size_t pos = mapName.find("_");
+	while(pos != std::string::npos)
+	{
+		mapName.replace(pos, 1, " ");
+		pos = mapName.find("_");
+	}
+	mapHeader.setMapName(glob2FilenameToName(filename));
+	
+	return mapHeader;
+}
+
+
+
+GameHeader Engine::loadGameHeader(const std::string &filename)
+{
+	MapHeader mapHeader;
+	GameHeader gameHeader;
+	InputStream *stream = new BinaryInputStream(Toolkit::getFileManager()->openInputStreamBackend(filename));
+	if (stream->isEndOfStream())
+	{
+		std::cerr << "Engine::loadGameHeader : error, can't open file " << filename  << std::endl;
+	}
+	else
+	{
+		if (verbose)
+			std::cout << "Engine::loadGameHeader : loading map " << filename << std::endl;
+		mapHeader.load(stream);
+		bool validMapSelected = gameHeader.load(stream, mapHeader.getVersionMinor());
+		if (!validMapSelected)
+			std::cerr << "Engine::loadGameHeader : invalid game header for map " << filename << std::endl;
+	}
+	delete stream;
+	return gameHeader;
+
+}
+
+
+
 int Engine::initGame(MapHeader& mapHeader, GameHeader& gameHeader, bool setGameHeader, bool ignoreGUIData)
 {
 	if (!gui.loadFromHeaders(mapHeader, gameHeader, setGameHeader, ignoreGUIData))
@@ -516,49 +630,52 @@ bool Engine::loadGame(const std::string &filename)
 
 
 
-MapHeader Engine::loadMapHeader(const std::string &filename)
+MapHeader Engine::chooseRandomMap()
 {
-	MapHeader mapHeader;
-	InputStream *stream = new BinaryInputStream(Toolkit::getFileManager()->openInputStreamBackend(filename));
-	if (stream->isEndOfStream())
+	std::vector<std::string> maps;
+
+	std::string fullDir = "maps";
+
+	// we add the other files
+	if (Toolkit::getFileManager()->initDirectoryListing(fullDir.c_str(), "map", false))
 	{
-		std::cerr << "Engine::loadMapHeader : error, can't open file " << filename  << std::endl;
+		const char* fileName;
+		while ((fileName = (Toolkit::getFileManager()->getNextDirectoryEntry())) != NULL)
+		{
+			std::string fullFileName = fullDir + DIR_SEPARATOR + fileName;
+			maps.push_back(fullFileName);
+		}
 	}
-	else
-	{
-		if (verbose)
-			std::cout << "Engine::loadMapHeader : loading map " << filename << std::endl;
-		bool validMapSelected = mapHeader.load(stream);
-		if (!validMapSelected)
-			std::cerr << "Engine::loadMapHeader : invalid map header for map " << filename << std::endl;
-	}
-	delete stream;
-	return mapHeader;
+	
+	int number = syncRand() % maps.size();
+	
+	return loadMapHeader(maps[number]);
 }
 
 
 
-GameHeader Engine::loadGameHeader(const std::string &filename)
+GameHeader Engine::createRandomGame(int numberOfTeams)
 {
-	MapHeader mapHeader;
 	GameHeader gameHeader;
-	InputStream *stream = new BinaryInputStream(Toolkit::getFileManager()->openInputStreamBackend(filename));
-	if (stream->isEndOfStream())
+	int count = 0;
+	for (int i=0; i<numberOfTeams+1; i++)
 	{
-		std::cerr << "Engine::loadGameHeader : error, can't open file " << filename  << std::endl;
+		int teamColor=(i % numberOfTeams);
+		if (i==0)
+		{
+			gameHeader.getBasePlayer(count) = BasePlayer(0, globalContainer->getUsername().c_str(), teamColor, BasePlayer::P_LOCAL);
+		}
+		else
+		{
+			AI::ImplementitionID iid=static_cast<AI::ImplementitionID>(syncRand() % 5 + 1);
+			FormatableString name("%0 %1");
+			name.arg(AI::getAIText(iid)).arg(i-1);
+			gameHeader.getBasePlayer(count) = BasePlayer(i, name.c_str(), teamColor, Player::playerTypeFromImplementitionID(iid));
+		}
+		count+=1;
 	}
-	else
-	{
-		if (verbose)
-			std::cout << "Engine::loadGameHeader : loading map " << filename << std::endl;
-		mapHeader.load(stream);
-		bool validMapSelected = gameHeader.load(stream, mapHeader.getVersionMinor());
-		if (!validMapSelected)
-			std::cerr << "Engine::loadGameHeader : invalid game header for map " << filename << std::endl;
-	}
-	delete stream;
+	gameHeader.setNumberOfPlayers(count);
 	return gameHeader;
-
 }
 
 
