@@ -52,6 +52,7 @@
 #include "DynamicClouds.h"
 
 #include "TextStream.h"
+#include "FertilityCalculatorDialog.h"
 
 #define BULLET_IMGID 0
 
@@ -85,6 +86,8 @@ Game::~Game()
 		}
 	}
 	
+	overlayAlphas.resize(0);
+
 	clearGame();
 }
 
@@ -102,28 +105,24 @@ void Game::init(GameGUI *gui, MapEdit* edit)
 		teams[i]=NULL;
 		players[i]=NULL;
 	}
+	clearGame();
 	
-	setSyncRandSeed();
-	fprintf(logFile, "setSyncRandSeed(%d, %d, %d)\n", getSyncRandSeedA(), getSyncRandSeedB(), getSyncRandSeedC());
 	
 	mouseX=0;
 	mouseY=0;
-	mouseUnit=NULL;
-	selectedUnit=NULL;
-	selectedBuilding=NULL;
 
 	stepCounter=0;
-	totalPrestige=0;
 	prestigeToReach=0;
-	totalPrestigeReached=false;
-	isGameEnded=false;
 	
 	for (int i=0; i<32; i++)
 		ticksGameSum[i]=0;
+
+	anyPlayerWaitedTimeFor = 0;
+	maskAwayPlayer = 0;
 }
 
 
-
+/** Reset player and team lists, game end stuff and selection stuff. */
 void Game::clearGame()
 {
 	// Delete existing teams and players
@@ -148,6 +147,11 @@ void Game::clearGame()
 	totalPrestige=0;
 	totalPrestigeReached=false;
 	isGameEnded=false;
+
+	// Clears selections
+	mouseUnit = NULL;
+	selectedUnit = NULL;
+	selectedBuilding = NULL;
 }
 
 
@@ -165,28 +169,28 @@ void Game::setMapHeader(const MapHeader& newMapHeader)
 
 void Game::setGameHeader(const GameHeader& newGameHeader)
 {
-	// set the base players
-	for (int i=0; i<gameHeader.getNumberOfPlayers(); i++)
-		delete players[i];
-
-	gameHeader = newGameHeader;
-
 	for (int i=0; i<mapHeader.getNumberOfTeams(); ++i)
 	{
 		teams[i]->playersMask=0;
 		teams[i]->numberOfPlayer=0;
 	}
 
-	for (int i=0; i<gameHeader.getNumberOfPlayers(); i++)
+	for (int i=0; i<newGameHeader.getNumberOfPlayers(); i++)
 	{
-		if(players[i])
+		//Don't change AI's
+		if(gameHeader.getBasePlayer(i).type < BasePlayer::P_AI)
+		{
 			delete players[i];
-		players[i]=new Player();
-		players[i]->setBasePlayer(&gameHeader.getBasePlayer(i), teams);
+			players[i]=new Player();
+			players[i]->setBasePlayer(&newGameHeader.getBasePlayer(i), teams);
+		}
 		teams[players[i]->teamNumber]->numberOfPlayer+=1;
 		teams[players[i]->teamNumber]->playersMask|=(1<<i);
 	}
 
+	setSyncRandSeed(newGameHeader.getRandomSeed());
+
+	gameHeader = newGameHeader;
 	anyPlayerWaited=false;
 }
 
@@ -498,6 +502,7 @@ void Game::executeOrder(boost::shared_ptr<Order> order, int localPlayer)
 				assert(false);
 			map.updateForbiddenGradient(oaa->teamNumber);
 			map.updateGuardAreasGradient(oaa->teamNumber);
+			map.updateClearAreasGradient(oaa->teamNumber);
 		}
 		break;
 		case ORDER_ALTERATE_GUARD_AREA:
@@ -591,6 +596,7 @@ void Game::executeOrder(boost::shared_ptr<Order> order, int localPlayer)
 			}
 			else
 				assert(false);
+			map.updateClearAreasGradient(oaa->teamNumber);
 		}
 		break;
 		case ORDER_MODIFY_SWARM:
@@ -699,6 +705,23 @@ void Game::executeOrder(boost::shared_ptr<Order> order, int localPlayer)
 		case ORDER_PLAYER_QUIT_GAME:
 		{
 			boost::shared_ptr<PlayerQuitsGameOrder> pqgo=boost::static_pointer_cast<PlayerQuitsGameOrder>(order);
+
+			bool found = false;
+			for(int i=0; i<32; ++i)
+			{
+				if(i!=pqgo->player && players[i])
+				{
+					if(players[i]->teamNumber == players[pqgo->player]->teamNumber)
+					{
+						found = true;
+					}
+				}
+			}
+			if(! found)
+			{
+				teams[players[pqgo->player]->teamNumber]->isAlive = false;
+			}
+
 			players[pqgo->player]->makeItAI(AI::NONE);
 			gameHeader.getBasePlayer(pqgo->player).makeItAI(AI::NONE);
 			fprintf(logFile, "ORDER_PLAYER_QUIT_GAME");
@@ -749,6 +772,7 @@ void Game::setAIAlliance(void)
 			printf("AI mask : %x\n", aiMask);
 				
 		// ally them together
+		
 		for (int i=0; i<mapHeader.getNumberOfTeams(); i++)
 			if (teams[i]->type == BaseTeam::T_AI)
 			{
@@ -773,6 +797,23 @@ void Game::setAIAlliance(void)
 	}
 }
 
+
+
+void Game::setAIFFA(void)
+{
+	// free for all on AI side
+	for (int i=0; i<mapHeader.getNumberOfTeams(); i++)
+	{
+		if (teams[i]->type == BaseTeam::T_AI)
+		{
+			teams[i]->allies = teams[i]->me;
+			teams[i]->enemies = ~teams[i]->allies;
+		}
+	}
+}
+
+
+
 bool Game::load(GAGCore::InputStream *stream)
 {
 	assert(stream);
@@ -820,18 +861,33 @@ bool Game::load(GAGCore::InputStream *stream)
 		return false;
 	}
 
-	///Load the step counter and random seeds	
+	///Load the step counter
 	stepCounter = stream->readUint32("stepCounter");
-	setSyncRandSeedA(stream->readUint32("SyncRandSeedA"));
-	setSyncRandSeedB(stream->readUint32("SyncRandSeedB"));
-	setSyncRandSeedC(stream->readUint32("SyncRandSeedC"));
-
-	stream->read(signature, 4, "signatureAfterSyncRand");
-	if (memcmp(signature,"GaSy", 4)!=0)
+	
+	if(versionMinor < 64)
 	{
-		fprintf(logFile, "Signature missmatch after Game::load sync rand\n");
-		stream->readLeaveSection();
-		return false;
+		///Load random seeds, these are no longer used
+		stream->readUint32("SyncRandSeedA");
+		stream->readUint32("SyncRandSeedB");
+		stream->readUint32("SyncRandSeedC");
+
+		stream->read(signature, 4, "signatureAfterSyncRand");
+		if (memcmp(signature,"GaSy", 4)!=0)
+		{
+			fprintf(logFile, "Signature missmatch after Game::load sync rand\n");
+			stream->readLeaveSection();
+			return false;
+		}
+	}
+	else
+	{
+		stream->read(signature, 4, "signatureBeforeTeams");
+		if (memcmp(signature,"GaBt", 4)!=0)
+		{
+			fprintf(logFile, "Signature missmatch before Game::load teams \n");
+			stream->readLeaveSection();
+			return false;
+		}
 	}
 
 	///Load teams
@@ -892,9 +948,8 @@ bool Game::load(GAGCore::InputStream *stream)
 		teams[i]->update();
 	}
 	
-	///Check teams integrity
-	for (int i=0; i<mapHeader.getNumberOfTeams(); i++)
-		teams[i]->integrity();
+	// Check integrity of loaded game
+	integrity();
 	
 	// Now load the map script
 	if (!script.load(stream, this))
@@ -910,18 +965,75 @@ bool Game::load(GAGCore::InputStream *stream)
 	prestigeToReach = std::max(MIN_MAX_PRESIGE, mapHeader.getNumberOfTeams()*TEAM_MAX_PRESTIGE);
 
 	stream->readLeaveSection();
+	
+	///versions less than 63 did not have fertility computed with the map, but computed it live.
+	///compute it now
+	if(mapHeader.getVersionMinor() < 63)
+	{
+	    if(globalContainer->runNoX)
+	    {
+    	    std::queue<boost::shared_ptr<FertilityCalculatorThreadMessage> > incoming;
+    	    boost::recursive_mutex incomingMutex;
+    	    FertilityCalculatorThread calculator(map, incoming, incomingMutex);
+    	    calculator();
+	    }
+	    else
+	    {
+    		FertilityCalculatorDialog dialog(globalContainer->gfx, map);
+	    	dialog.execute();
+	    }
+	}
+	
 	return true;
 }
 
+void Game::integrity(void)
+{
+	///Check teams integrity
+	for (int i=0; i<mapHeader.getNumberOfTeams(); i++)
+		teams[i]->integrity();
+	
+	///Check that all ID do point to existing objects
+	for (int y=0; y<map.getH(); y++)
+		for (int x=0; x<map.getW(); x++)
+		{
+			Case& c = map.getCase(x, y);
+			if (c.building != NOGBID)
+			{
+				int tid = Building::GIDtoTeam(c.building);
+				assert(teams[tid]);
+				assert(teams[tid]->myBuildings[Building::GIDtoID(c.building)]);
+			}
+			if (c.groundUnit != NOGUID)
+			{
+				int tid = Unit::GIDtoTeam(c.groundUnit);
+				assert(teams[tid]);
+				assert(teams[tid]->myUnits[Unit::GIDtoID(c.groundUnit)]);
+			}
+			if (c.airUnit != NOGUID)
+			{
+				int tid = Unit::GIDtoTeam(c.airUnit);
+				assert(teams[tid]);
+				assert(teams[tid]->myUnits[Unit::GIDtoID(c.airUnit)]);
+			}
+		}
+}
+
 void Game::save(GAGCore::OutputStream *stream, bool fileIsAMap, const std::string& name)
 {
 	assert(stream);
 	stream->writeEnterSection("Game");
-
+	if(dynamic_cast<GAGCore::BinaryOutputStream*>(stream))
+	{
+		dynamic_cast<GAGCore::BinaryOutputStream*>(stream)->enableSHA1();
+	}
+    
 	///Save the two headers, record the position in the file because mapHeader will
 	///will need to be overwritten with the mapOffset known
 	Uint32 mapHeaderOffset = stream->getPosition();
 	mapHeader.setMapName(name);
+	mapHeader.setIsSavedGame(!fileIsAMap);
+	mapHeader.resetGameSHA1();
 	
 	for (int i=0; i<mapHeader.getNumberOfTeams(); ++i)
 	{
@@ -941,10 +1053,7 @@ void Game::save(GAGCore::OutputStream *stream, bool fileIsAMap, const std::strin
 	///Save basic informations
 	stream->write("GaBe", 4, "signatureStart");
 	stream->writeUint32(stepCounter, "stepCounter");
-	stream->writeUint32(getSyncRandSeedA(), "SyncRandSeedA");
-	stream->writeUint32(getSyncRandSeedB(), "SyncRandSeedB");
-	stream->writeUint32(getSyncRandSeedC(), "SyncRandSeedC");
-	stream->write("GaSy", 4, "signatureAfterSyncRand");
+	stream->write("GaBt", 4, "signatureBeforeTeams");
 
 	///Save teams
 	stream->writeEnterSection("teams");
@@ -980,7 +1089,16 @@ void Game::save(GAGCore::OutputStream *stream, bool fileIsAMap, const std::strin
 	
 	///Save the campaign text
 	stream->writeText(campaignText, "campaignText");
-	
+
+	Uint8 sha1[20];
+	for(int i=0; i<20; ++i)
+		sha1[i]=0;
+	if(dynamic_cast<GAGCore::BinaryOutputStream*>(stream))
+	{
+		dynamic_cast<GAGCore::BinaryOutputStream*>(stream)->finishSHA1(sha1);
+	}
+	mapHeader.setGameSHA1(sha1);
+
 	///Overwrite the MapHeader. This is done after the map
 	///offset has been set.
 	if (stream->canSeek())
@@ -1069,8 +1187,15 @@ void Game::wonSyncStep(void)
 			bool isOtherAlive=false;
 			for (int j=0; j<mapHeader.getNumberOfTeams(); j++)
 			{
-				if ((j!=i) && (!( ((teams[i]->me) & (teams[j]->allies)) /*&& ((teams[j]->me) & (teams[i]->allies))*/ )) && (teams[j]->isAlive))
-					isOtherAlive=true;
+				if(j!=i)
+				{
+					Uint32 playerToMeAllyMask = teams[i]->me & teams[j]->allies;
+					Uint32 meToPlayerAllyMask = teams[j]->me & teams[i]->allies;
+					if((playerToMeAllyMask==0 || meToPlayerAllyMask==0) && teams[j]->isAlive)
+					{
+						isOtherAlive=true;
+					}
+				}
 			}
 			teams[i]->hasWon |= !isOtherAlive;
 			isGameEnded |= teams[i]->hasWon;
@@ -1151,6 +1276,7 @@ void Game::syncStep(Sint32 localTeam)
 		Sint32 endTick=SDL_GetTicks();
 		ticksGameSum[stepCounter&31]+=endTick-startTick;
 		stepCounter++;
+		anyPlayerWaitedTimeFor+=1;
 	}
 }
 
@@ -1180,6 +1306,8 @@ void Game::addTeam(int pos)
 		prestigeToReach = std::max(MIN_MAX_PRESIGE, pos*TEAM_MAX_PRESTIGE);
 		
 		map.addTeam();
+		
+		script.addTeam();
 	}
 	else
 		assert(false);
@@ -1205,6 +1333,7 @@ void Game::removeTeam(int pos)
 			teams[i]->setCorrectColor(((float)i*360.0f)/(float)mapHeader.getNumberOfTeams());
 
 		map.removeTeam();
+		script.removeTeam(pos);
 		teams[pos]=NULL;
 	}
 }
@@ -1254,6 +1383,9 @@ Unit *Game::addUnit(int x, int y, int team, Sint32 typeNum, int level, int delta
 
 	UnitType *ut=teams[team]->race.getUnitType(typeNum, level);
 
+	x = (x + map.getW()) % map.getW();
+	y = (y + map.getH()) % map.getH();
+
 	bool fly=ut->performance[FLY];
 	bool free;
 	if (fly)
@@ -1592,7 +1724,7 @@ void Game::drawUnit(int x, int y, Uint16 gid, int viewportX, int viewportY, int
 
 	// draw unit
 	Sprite *unitSprite = unit->skin->sprite;
-	unitSprite->setBaseColor(teams[team]->colorR, teams[team]->colorG, teams[team]->colorB);
+	unitSprite->setBaseColor(teams[team]->color);
 	int decX = (unitSprite->getW(imgid)-32)>>1;
 	int decY = (unitSprite->getH(imgid)-32)>>1;
 	globalContainer->gfx->drawSprite(px-decX, py-decY, unitSprite, imgid);
@@ -1660,23 +1792,7 @@ void Game::drawUnit(int x, int y, Uint16 gid, int viewportX, int viewportY, int
 		if ((unit->performance[HARVEST]) && (unit->caryedRessource>=0))
 			globalContainer->gfx->drawSprite(px+24, py, globalContainer->ressourceMini, unit->caryedRessource);
 	}
-	if (((drawOptions & DRAW_PATH_LINE) != 0) && (unit->owner->sharedVisionOther & teams[localTeam]->me))
-		if (unit->validTarget)
-		/*TODO: remove this comment when we see the new code works
-			if (unit->displacement==Unit::DIS_GOING_TO_FLAG
-			|| unit->displacement==Unit::DIS_GOING_TO_RESSOURCE
-			|| unit->displacement==Unit::DIS_GOING_TO_BUILDING
-			|| (unit->displacement==Unit::DIS_ATTACKING_AROUND && unit->movement==Unit::MOV_GOING_DXDY))*/
-		{
-			int lsx, lsy, ldx, ldy;
-			lsx=px+16;
-			lsy=py+16;
-			map.mapCaseToDisplayableVector(unit->targetX, unit->targetY, &ldx, &ldy, viewportX, viewportY, screenW, screenH);
-			if (globalContainer->settings.optionFlags & GlobalContainer::OPTION_LOW_SPEED_GFX)
-				globalContainer->gfx->drawLine(lsx, lsy, ldx+16, ldy+16, 250, 250, 250);
-			else
-				globalContainer->gfx->drawLine(lsx, lsy, ldx+16, ldy+16, 250, 250, 250, 128);
-		}
+
 	if (drawOptions & DRAW_ACCESSIBILITY)
 	{
 		std::ostringstream oss;
@@ -1788,7 +1904,9 @@ inline void Game::drawMapRessources(int left, int top, int right, int bot, int v
 
 inline void Game::drawMapGroundUnits(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
 {
-	mouseUnit=NULL;//??
+	//Reset the mouse unit to NULL, as this time arround there may not be a unit
+	//under the mouse pointer
+	mouseUnit=NULL;
 	for (int y=top-1; y<=bot; y++)
 		for (int x=left-1; x<=right; x++)
 		{
@@ -1806,7 +1924,7 @@ inline void Game::drawMapDebugAreas(int left, int top, int right, int bot, int s
 			{
 				//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, ((AICastor *)players[1]->ai->aiImplementation)->wheatCareMap[0][(x+viewportX)+(y+viewportY)*map.w]);
 				//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, ((AICastor *)players[1]->ai->aiImplementation)->notGrassMap[(x+viewportX)+(y+viewportY)*map.w]);
-//				globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.getExplored(x+viewportX, y+viewportY, 0));
+//				globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.guardAreasGradient[0][1][(x+viewportX)+(y+viewportY)*map.w]);
 //				globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, ((Nicowar::AINicowar*)players[3]->ai->aiImplementation)->getGradientManager().getGradient(Nicowar::Gradient::VillageCenter, Nicowar::Gradient::Resource).getHeight(x+viewportX, y+viewportY));
 				//((AICastor *)players[0].ai->aiImplementation)->wheatCareMap
 			}
@@ -1826,10 +1944,11 @@ inline void Game::drawMapDebugAreas(int left, int top, int right, int bot, int s
 	if (false)
 	{
 		assert(teams[0]);
-		Building *b=teams[0]->myBuildings[5];
+		Building *b=teams[0]->myBuildings[0];
 		if (b)
 			for (int y=top-1; y<=bot; y++)
 				for (int x=left-1; x<=right; x++)
+				{
 					if (map.warpDistMax(b->posX, b->posY, x+viewportX, y+viewportY)<16)
 					{
 						//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, "%d", map.getGradient(0, 6, 1, x+viewportX, y+viewportY));
@@ -1844,6 +1963,7 @@ inline void Game::drawMapDebugAreas(int left, int top, int right, int bot, int s
 						//globalContainer->gfx->drawString((x<<5), (y<<5)+16, globalContainer->littleFont, "%d", x+viewportX-b->posX+16);
 						//globalContainer->gfx->drawString((x<<5)+16, (y<<5)+16, globalContainer->littleFont, "%d", y+viewportY-b->posY+16);
 					}
+				}
 	}
 	
 	// We draw debug area:
@@ -1907,7 +2027,7 @@ inline void Game::drawMapDebugAreas(int left, int top, int right, int bot, int s
 	}
 }
 
-inline void Game::drawMapGroundBuildings(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+inline void Game::drawMapGroundBuildings(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions, std::set<Building*> *visibleBuildings)
 {
 	std::set<Building *, BuildingPosComp> buildingList;
 	for (int y=top-1; y<=bot; y++)
@@ -1928,7 +2048,11 @@ inline void Game::drawMapGroundBuildings(int left, int top, int right, int bot,
 					|| Building::GIDtoTeam(gid)==localTeam
 					|| (building->seenByMask & teams[localTeam]->me)
 					|| map.isFOWDiscovered(x+viewportX, y+viewportY, teams[localTeam]->me))
+				{
 					buildingList.insert(building);
+					if (visibleBuildings)
+						visibleBuildings->insert(building);
+				}
 			}
 		}
 	
@@ -1982,7 +2106,7 @@ inline void Game::drawMapGroundBuildings(int left, int top, int right, int bot,
 		Sprite *buildingSprite = type->gameSpritePtr;
 		dx = (type->width<<5)-buildingSprite->getW(imgid);
 		dy = (type->height<<5)-buildingSprite->getH(imgid);
-		buildingSprite->setBaseColor(team->colorR, team->colorG, team->colorB);
+		buildingSprite->setBaseColor(team->color);
 
 		// draw building
 		globalContainer->gfx->drawSprite(x+dx, y+dy, buildingSprite, imgid);
@@ -2282,7 +2406,7 @@ inline void Game::drawMapBulletsExplosionsDeathAnimations(int left, int top, int
 				int decY = globalContainer->deathAnimation->getH(frame)>>1;
 				Team *team = (*it)->team;
 				
-				globalContainer->deathAnimation->setBaseColor(team->colorR, team->colorG, team->colorB);
+				globalContainer->deathAnimation->setBaseColor(team->color);
 				globalContainer->gfx->drawSprite(x+16-decX, y+16-decY-frame, globalContainer->deathAnimation, frame);
 			}
 		}
@@ -2356,16 +2480,17 @@ inline void Game::drawMapOverlayMaps(int left, int top, int right, int bot, int
 			overlayColor=Color(0, 0, 192);
 		if(overlays->getOverlayType() == OverlayArea::Fertility)
 			overlayColor=Color(0, 192, 128);
-		int width = (right - left) + 1;
-		int height = (bot - top) + 1;
+		///Both width and height have +2 to cover half-squares arround the edge of the viewport
+		int width = (right - left) + 2;
+		int height = (bot - top) + 2;
 
 		overlayAlphas.resize(width * height);
-		for (int y=top-1; y<=bot; y++)
+		for (int y=0; y<height; y++)
 		{
-			for (int x=left-1; x<=right; x++)
+			for (int x=0; x<width; x++)
 			{
-				int rx=(x+viewportX)%map.getW();
-				int ry=(y+viewportY)%map.getH();
+				int rx=(x+viewportX-1+map.getW())%map.getW();
+				int ry=(y+viewportY-1+map.getH())%map.getH();
 				if(!edit && !map.isMapDiscovered(rx, ry, teams[localTeam]->me))
 					continue;
 				if(overlays->getValue(rx, ry))
@@ -2379,22 +2504,91 @@ inline void Game::drawMapOverlayMaps(int left, int top, int right, int bot, int
 		
 		///This is to correct OpenGL's blending not beeing offset correctly to line up with the map tiles
 		if(globalContainer->gfx->getOptionFlags() & GraphicContext::USEGPU)
-			globalContainer->gfx->drawAlphaMap(overlayAlphas, width, height, 16, 16, 32, 32, overlayColor);
+			globalContainer->gfx->drawAlphaMap(overlayAlphas, width, height, -16, -16, 32, 32, overlayColor);
 		else
-			globalContainer->gfx->drawAlphaMap(overlayAlphas, width, height, 0, 0, 32, 32, overlayColor);
+			globalContainer->gfx->drawAlphaMap(overlayAlphas, width, height, -32, -32, 32, 32, overlayColor);
 	}
 }
 
 
+
+inline void Game::drawUnitPathLines(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
+	for(int i=0; i<1024; ++i)
+	{
+		Unit *unit=teams[localTeam]->myUnits[i];
+		if (unit)
+		{
+			drawUnitPathLine(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions, unit);
+		}
+	}
+}
+
+
+
+inline void Game::drawUnitPathLine(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions, Unit* unit)
+{
+	if (((drawOptions & DRAW_PATH_LINE) != 0) && (unit->owner->sharedVisionOther & teams[localTeam]->me))
+	{
+		if (unit->validTarget)
+		{
+			if(isOnScreen(left,top,right,bot,viewportX,viewportY,unit->posX,unit->posY) || isOnScreen(left,top,right,bot,viewportX,viewportY,unit->targetX,unit->targetY))
+			{
+				int px, py;
+				map.mapCaseToDisplayableVector(unit->posX, unit->posY, &px, &py, viewportX, viewportY, sw, sh);
+				int deltaLeft=255-unit->delta;
+				if (unit->action<BUILD)
+				{
+					px-=(unit->dx*deltaLeft)>>3;
+					py-=(unit->dy*deltaLeft)>>3;
+				}
+			
+			
+				int lsx, lsy, ldx, ldy;
+				map.mapCaseToDisplayableVector(unit->targetX, unit->targetY, &ldx, &ldy, viewportX, viewportY, sw, sh);
+				lsx=px+16;
+				lsy=py+16;
+				if (globalContainer->settings.optionFlags & GlobalContainer::OPTION_LOW_SPEED_GFX)
+					globalContainer->gfx->drawLine(lsx, lsy, ldx+16, ldy+16, 250, 250, 250);
+				else
+					globalContainer->gfx->drawLine(lsx, lsy, ldx+16, ldy+16, 250, 250, 250, 128);
+			}
+		}
+	}
+}
+
+
+
 float Game::interpolateValues(float a, float b, float x)
 {
 	float ft = 3.141592653f * x;
 	float f = (1.0f - std::cos(ft)) * 0.5f;
 	return  a*(1.0-f) + b*f;
 }
+
+
+
+inline bool Game::isOnScreen(int left, int top, int right, int bot, int viewportX, int viewportY, int x, int y)
+{
+
+	left += viewportX;
+	right += viewportX;
+	top += viewportY;
+	bot += viewportY;
+	
+	if((x >= left-1 && x <= right) || (x+map.getW() >= left-1 && x+map.getW() <= right))
+	{
+		if((y >= top-1 && y <= bot) || (y+map.getH() >= top-1 && y+map.getH() <= bot))
+		{
+			return true;
+		}
+	}
+	return false;
+}
+
 	
 
-void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions, std::set<Building*> *visibleBuildings)
 {
 	static int time = 0;
 	static DynamicClouds ds(&globalContainer->settings);
@@ -2409,7 +2603,7 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 	drawMapRessources(left, top, right, bot, viewportX, viewportY, localTeam, drawOptions);
 	drawMapGroundUnits(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
 	drawMapDebugAreas(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
-	drawMapGroundBuildings(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
+	drawMapGroundBuildings(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions, visibleBuildings);
 	drawMapAirUnits(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
 	if((drawOptions & DRAW_SCRIPT_AREAS) != 0)
 		drawMapScriptAreas(left, top, right, bot, viewportX, viewportY);
@@ -2425,6 +2619,7 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 	drawMapFogOfWar(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
 	drawMapAreas(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
 	drawMapOverlayMaps(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
+	drawUnitPathLines(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
 	
 	// draw cloud overlay if we are in high quality
 	if ((globalContainer->settings.optionFlags & GlobalContainer::OPTION_LOW_SPEED_GFX) == 0)
@@ -2447,7 +2642,7 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 
 		// all flags are hued:
 		Sprite *buildingSprite = type->gameSpritePtr;
-		buildingSprite->setBaseColor(teams[team]->colorR, teams[team]->colorG, teams[team]->colorB);
+		buildingSprite->setBaseColor(teams[team]->color);
 		globalContainer->gfx->drawSprite(x, y, buildingSprite, imgid);
 
 		// flag circle:
@@ -2527,7 +2722,19 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 
 void Game::setWaitingOnMask(Uint32 mask)
 {
+	Uint32 oldMask = maskAwayPlayer;
 	maskAwayPlayer = mask;
+
+	if(mask != 0)
+	{
+		if(oldMask == 0)
+			anyPlayerWaitedTimeFor = 0;
+		anyPlayerWaited = true;
+	}
+	else
+	{
+		anyPlayerWaited = false;
+	}
 }
 
 
@@ -2549,7 +2756,7 @@ void Game::dumpAllData(const std::string& file)
 
 
 
-Uint32 Game::checkSum(std::vector<Uint32> *checkSumsVector, std::vector<Uint32> *checkSumsVectorForBuildings, std::vector<Uint32> *checkSumsVectorForUnits)
+Uint32 Game::checkSum(std::vector<Uint32> *checkSumsVector, std::vector<Uint32> *checkSumsVectorForBuildings, std::vector<Uint32> *checkSumsVectorForUnits, bool heavy)
 {
 	Uint32 cs=0;
 
@@ -2586,34 +2793,25 @@ Uint32 Game::checkSum(std::vector<Uint32> *checkSumsVector, std::vector<Uint32>
 	
 	cs=(cs<<31)|(cs>>1);
 	
-	bool heavy=false;
 	for (int i=0; i<gameHeader.getNumberOfPlayers(); i++)
+	{
 		if (players[i]->type==BasePlayer::P_IP)
 		{
 			heavy=true;
 			break;
 		}
+	}
 	Uint32 mapCs=map.checkSum(heavy);
 	cs^=mapCs;
 	if (checkSumsVector)
 		checkSumsVector->push_back(mapCs);// [3+t*20+p*2]
-	
-	cs=(cs<<31)|(cs>>1);
-
-	Uint32 syncRandCs=0;
-	syncRandCs^=getSyncRandSeedA();
-	syncRandCs^=getSyncRandSeedB();
-	syncRandCs^=getSyncRandSeedC();
-	cs^=syncRandCs;
-	if (checkSumsVector)
-		checkSumsVector->push_back(syncRandCs);// [4+t*20+p*2]
 
 	cs=(cs<<31)|(cs>>1);
 	
 	Uint32 scriptCs=script.checkSum();
 	cs^=scriptCs;
 	if (checkSumsVector)
-		checkSumsVector->push_back(scriptCs);// [5+t*20+p*2]
+		checkSumsVector->push_back(scriptCs);// [4+t*20+p*2]
 	
 	return cs;
 }
@@ -2637,20 +2835,18 @@ Team *Game::getTeamWithMostPrestige(void)
 
 std::string glob2FilenameToName(const std::string& filename)
 {
-	GAGCore::InputStream *stream = new GAGCore::BinaryInputStream(GAGCore::Toolkit::getFileManager()->openInputStreamBackend(filename.c_str()));
-	if (stream->isEndOfStream())
-	{
-		delete stream;
-	}
+	std::string mapName;
+	if(filename.find(".game")!=std::string::npos)
+		mapName=filename.substr(filename.find("/")+1, filename.size()-6-filename.find("/"));
 	else
+		mapName=filename.substr(filename.find("/")+1, filename.size()-5-filename.find("/"));
+	size_t pos = mapName.find("_");
+	while(pos != std::string::npos)
 	{
-		MapHeader tempHeader;
-		bool res = tempHeader.load(stream);
-		delete stream;
-		if (res)
-			return tempHeader.getMapName();
+		mapName.replace(pos, 1, " ");
+		pos = mapName.find("_");
 	}
-	return "";
+	return mapName;
 }
 
 template<typename It, typename T>
