@@ -21,6 +21,8 @@
 #define __GAME_GUI_DIALOG_H
 
 #include <GUIBase.h>
+
+
 using namespace GAGGUI;
 namespace GAGGUI
 {
@@ -29,6 +31,7 @@ namespace GAGGUI
 	class Text;
 }
 class GameGUI;
+class GameHeader;
 
 class InGameMainScreen:public OverlayScreen
 {
@@ -78,6 +81,7 @@ public:
 	};
 
 public:
+	Text *texts[16];
 	OnOffButton *alliance[16];
 	OnOffButton *normalVision[16];
 	OnOffButton *foodVision[16];
@@ -89,6 +93,7 @@ public:
 	InGameAllianceScreen(GameGUI *gameGUI);
 	virtual ~InGameAllianceScreen() { }
 	virtual void onAction(Widget *source, Action action, int par1, int par2);
+	int countNumberPlayersForLocalTeam(GameHeader& gameHeader, int localteam);
 	Uint32 getAlliedMask(void);
 	Uint32 getEnemyMask(void);
 	Uint32 getExchangeVisionMask(void);
@@ -111,11 +116,13 @@ public:
 
 public:
 	Selector *musicVol;
+	Selector *voiceVol;
 	OnOffButton* mute;
 	Text *musicVolText;
+	Text *voiceVolText;
 public:
 	InGameOptionScreen(GameGUI *gameGUI);
-	virtual ~InGameOptionScreen() { }
+	~InGameOptionScreen();
 	virtual void onAction(Widget *source, Action action, int par1, int par2);
 };
 
