@@ -30,6 +30,7 @@
 #include "GlobalContainer.h"
 #include "Header.h"
 #include "IntBuildingType.h"
+#include "KeyboardManager.h"
 #include "LogFileManager.h"
 #include "MapEditKeyActions.h"
 #include "NonANSICStdWrapper.h"
@@ -78,6 +79,10 @@ GlobalContainer::GlobalContainer(void)
 	hostServerUserName[0] = 0;
 	hostServerPassWord[0] = 0;
 	
+	runTestGames=false;
+	automaticEndingGame=false;
+	automaticEndingSteps=-1;
+	
 	gfx = NULL;
 	mix = NULL;
 	terrain = NULL;
@@ -92,6 +97,7 @@ GlobalContainer::GlobalContainer(void)
 	littleFont = NULL;
 	
 	voiceRecorder = NULL;
+	automaticGameGlobalEndConditions=false;
 
 	assert((int)USERNAME_MAX_LENGTH==(int)BasePlayer::MAX_NAME_LENGTH);
 }
@@ -148,7 +154,8 @@ void GlobalContainer::parseArgs(int argc, char *argv[])
 			{
 				runNoXGameName = argv[i + 1];
 				runNoX = true;
-				good &= (sscanf(argv[i + 2], "%d", &runNoXCountSteps) == 1);
+				automaticEndingGame = true;
+				good &= (sscanf(argv[i + 2], "%d", &automaticEndingSteps) == 1);
 				good &= (sscanf(argv[i + 3], "%d", &runNoXCountRuns) == 1);
 				i += 3;
 			}
@@ -170,6 +177,21 @@ void GlobalContainer::parseArgs(int argc, char *argv[])
 			hostServer=true;
 			continue;
 		}
+		if (strcmp(argv[i], "-test-games")==0)
+		{
+			runTestGames=true;
+			automaticEndingGame = true;
+			automaticGameGlobalEndConditions=true;
+			continue;
+		}
+		if (strcmp(argv[i], "-test-games-nox")==0)
+		{
+			runTestGames=true;
+			automaticEndingGame = true;
+			runNoX=true;
+			automaticGameGlobalEndConditions=true;
+			continue;
+		}
 		if (strcmp(argv[i], "-host")==0 || strcmp(argv[i], "--host")==0)
 		{
 			if (i+3<argc)
@@ -319,6 +341,8 @@ void GlobalContainer::parseArgs(int argc, char *argv[])
 			printf("-daemon\t runs the YOG server\n");
 			printf("-nox <game file name> \t runs the game without using the X server\n");
 			printf("-textshot <directory>\t takes pictures of various translation texts as they are drawn on the screen, requires the convert command\n");
+			printf("-test-games\tCreates random games with AI and tests them");
+			printf("-test-games-nox\tCreates random games with AI and tests them, without gui");
 			printf("-vs <name>\tsave a videoshot as name\n");
 			printf("-version\tprint the version and exit\n");
 			exit(0);
@@ -429,6 +453,7 @@ void GlobalContainer::destroyProgressBar(void)
 
 void GlobalContainer::load(void)
 {
+
 	// load texts
 	if (!Toolkit::getStringTable()->load("data/texts.list.txt"))
 	{
@@ -444,15 +469,15 @@ void GlobalContainer::load(void)
 		exit(-1);
 	}
 	
-	Toolkit::getStringTable()->setLang(settings.defaultLanguage);
+	Toolkit::getStringTable()->setLang(Toolkit::getStringTable()->getLangCode(settings.language));
+		
 
 	if (!runNoX)
 	{
 		// create graphic context
-		gfx = Toolkit::initGraphic(settings.screenWidth, settings.screenHeight, settings.screenFlags);
+		gfx = Toolkit::initGraphic(settings.screenWidth, settings.screenHeight, settings.screenFlags, "Globulation 2", "glob 2");
 		gfx->setMinRes(640, 480);
 		//gfx->setQuality((settings.optionFlags & OPTION_LOW_SPEED_GFX) != 0 ? GraphicContext::LOW_QUALITY : GraphicContext::HIGH_QUALITY);
-		gfx->setCaption("Globulation 2", "glob 2");
 	}
 	
 	if (!runNoX)
@@ -460,7 +485,7 @@ void GlobalContainer::load(void)
 		initProgressBar();
 		
 		// create mixer
-		mix = new SoundMixer(settings.musicVolume, settings.mute);
+		mix = new SoundMixer(settings.musicVolume, settings.voiceVolume, settings.mute);
 		mix->loadTrack("data/zik/intro.ogg");
 		mix->loadTrack("data/zik/menu.ogg");
 		mix->loadTrack("data/zik/a1.ogg");
@@ -487,6 +512,17 @@ void GlobalContainer::load(void)
 	GameGUIKeyActions::init();
 	MapEditKeyActions::init();
 	
+	if(settings.version < 1)
+	{
+		KeyboardManager game(GameGUIShortcuts);
+		game.loadDefaultShortcuts();
+		game.saveKeyboardLayout();
+
+		KeyboardManager edit(MapEditShortcuts);
+		edit.loadDefaultShortcuts();
+		edit.saveKeyboardLayout();
+	}
+	
 	if (!runNoX)
 	{
 		updateLoadProgressBar(35);
