@@ -91,6 +91,7 @@ public:
 	Sprite *gamegui;
 	Sprite *brush;
 	Sprite *magiceffect;
+	Sprite *particles;
 	
 	UnitsSkins *unitsSkins;
 	
@@ -107,7 +108,11 @@ public:
 	bool runNoX;
 	std::string runNoXGameName;
 	int runNoXCountRuns; //!< The number of runs you want to repeat the no X run
-	int runNoXCountSteps; //!< The number of steps you want the no X game to run
+	bool automaticEndingGame;
+	int automaticEndingSteps;
+	bool automaticGameGlobalEndConditions; //! Set false if the automatic game will end if the local team wins/loses, true to wait for the entire game to finish
+	
+	bool runTestGames; //! runs test games
 	
 	bool hostServer;
 	char hostServerMapName[32];
