@@ -6,7 +6,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -31,6 +31,7 @@
 #include "Team.h"
 #include "TerrainType.h"
 #include "BitArray.h"
+#include "MapHeader.h"
 
 class Unit;
 
@@ -59,12 +60,13 @@ struct Case
 	Uint32 forbidden; // This is a mask, one bit by team, 1=forbidden, 0=allowed
 	///The difference between forbidden zone and hidden forbidden zone is that hidden forbidden zone
 	///is put there by the game engine and is not draw to the screen.
-	Uint32 hiddenForbidden; // This is a mask, one bit by team, 1=forbidden, 0=allowed
 	Uint32 guardArea; // This is a mask, one bit by team, 1=guard area, 0=normal
 	Uint32 clearArea; // This is a mask, one bit by team, 1=clear area, 0=normal
 
 	Uint16 scriptAreas; // This is also a mask. A single bit represents an area #n, on or off for the square
 	Uint8 canRessourcesGrow; // This is a boolean, it represents whether ressources are allowed to grow into this location.
+	
+	Uint16 fertility; // This is a value that represents the fertility of this square, the chance that wheat will grow on it
 };
 
 
@@ -74,7 +76,7 @@ struct Case
 */
 class Map
 {
-	static const bool verbose = true;
+	static const bool verbose = false;
 public:
 	//! Type of terrain (used for undermap)
 
@@ -93,10 +95,14 @@ public:
 	// !This call is needed to use the Map!
 	void setGame(Game *game);
 	//! Load a map from a stream and relink with associated game
-	bool load(GAGCore::InputStream *stream, SessionGame *sessionGame, Game *game=NULL);
+	bool load(GAGCore::InputStream *stream, MapHeader& header, Game *game=NULL);
 	//! Save a map
 	void save(GAGCore::OutputStream *stream);
 	
+	/// Load transitional map data. This is temporary map information that was saved by
+	/// the alpha 23 patch for transfering between map formats
+	void loadTransitional();
+	
 	// add & remove teams, used by the map editor and the random map generator
 	// Have to be called *after* session.numberOfTeam has been changed.
 	void addTeam(void);
@@ -129,13 +135,13 @@ public:
 	///Returns a normalized version of the x cordinate, taking into account that x cordinates wrap arround
 	int normalizeX(int x)
 	{
-		return (x+w)%w;
+		return (x + w) & wMask;
 	}
 	
 	///Returns a normalized version of the y cordinate, taking into account that y cordinates wrap arround
 	int normalizeY(int y)
 	{
-		return (y+h)%h;
+		return (y + h) & hMask;
 	}
 
 	//! Set map to discovered state at position (x, y) for all teams in sharedVision (mask).
@@ -309,7 +315,7 @@ public:
 	//Returns the combined forbidden and hidden foribidden masks
 	Uint32 getForbidden(int x, int y)
 	{
-		return cases[((y&hMask)<<wDec)+(x&wMask)].forbidden | cases[((y&hMask)<<wDec)+(x&wMask)].hiddenForbidden;
+		return cases[((y&hMask)<<wDec)+(x&wMask)].forbidden;
 	}
 	
 	Uint8 getExplored(int x, int y, int team)
@@ -340,19 +346,9 @@ public:
 	void removeForbidden(int x, int y, Uint32 teamNum)
 	{
 		Case& c=cases[((y&hMask)<<wDec)+(x&wMask)];
-		c.hiddenForbidden ^= c.forbidden &  Team::teamNumberToMask(teamNum);
-	}
-	
-	void addHiddenForbidden(int x, int y, Uint32 teamNum)
-	{
-		cases[((y&hMask)<<wDec)+(x&wMask)].hiddenForbidden |= Team::teamNumberToMask(teamNum);
+		c.forbidden ^= c.forbidden &  Team::teamNumberToMask(teamNum);
 	}
 
-	void removeHiddenForbidden(int x, int y, Uint32 teamNum)
-	{
-		Case& c=cases[((y&hMask)<<wDec)+(x&wMask)];
-		c.hiddenForbidden ^= c.hiddenForbidden &  Team::teamNumberToMask(teamNum);
-	}
 	
 	bool isWater(int x, int y)
 	{
@@ -376,6 +372,12 @@ public:
 		int t=getTerrain(x, y);
 		return ((t>=128)&&(t<128+16));
 	}
+	
+	bool hasSand(int x, int y)
+	{
+		int t=getTerrain(x, y);
+		return ((t>=16)&&(t<=255));
+	}
 
 	bool isRessource(int x, int y)
 	{
@@ -446,6 +448,22 @@ public:
 	//! Return true if unit has contact with enemy. If true, put contact direction in dx, dy
 	bool doesUnitTouchEnemy(Unit *unit, int *dx, int *dy);
 
+	//! Sets this particular clearing area location as claimed
+	void setClearingAreaClaimed(int x, int y, int teamNumber, int gid);
+	//! Sets this particular clearing area location as unclaimed
+	void setClearingAreaUnclaimed(int x, int y, int teamNumber);
+	//! Returns the gid if this clearing area is claimed, NOGUID otherwise
+	int isClearingAreaClaimed(int x, int y, int teamNumber);
+
+	//! Marks a particular square as containing an immobile unit
+	void markImmobileUnit(int x, int y, int teamNumber);
+	//! Clears a particular square of having an immobile unit
+	void clearImmobileUnit(int x, int y);
+	//! Returns true if theres an immobile unit on the square
+	bool isImmobileUnit(int x, int y);
+	//! Returns the team number of the immobile unit on the given square, 255 for none
+	Uint8 getImmobileUnit(int x, int y);
+
 	//! Return GID
 	Uint16 getGroundUnit(int x, int y) { return cases[((y&hMask)<<wDec)+(x&wMask)].groundUnit; }
 	Uint16 getAirUnit(int x, int y) { return cases[((y&hMask)<<wDec)+(x&wMask)].airUnit; }
@@ -517,7 +535,8 @@ public:
 		GT_BUILDING = 2,
 		GT_FORBIDDEN = 3,
 		GT_GUARD_AREA = 4,
-		GT_SIZE = 5
+		GT_CLEAR_AREA=5,
+		GT_SIZE = 6
 	};
 	
 	bool ressourceAvailable(int teamNumber, int ressourceType, bool canSwim, int x, int y);
@@ -534,6 +553,13 @@ public:
 		return gradient[((y&hMask)<<wDec)+(x&wMask)];
 	}
 	
+	Uint8 getClearingGradient(int teamNumber, bool canSwim, int x, int y)
+	{
+		Uint8 *gradient = clearAreasGradient[teamNumber][canSwim];
+		assert(gradient);
+		return gradient[((y&hMask)<<wDec)+(x&wMask)];
+	}
+	
 	void updateGlobalGradientSlow(Uint8 *gradient);
 	template<typename Tint> void updateGlobalGradientSlow(Uint8 *gradient);
 	
@@ -570,6 +596,8 @@ public:
 	bool pathfindForbidden(Uint8 *optionGradient, int teamNumber, bool canSwim, int x, int y, int *dx, int *dy, bool verbose);
 	//! Find the best direction toward gaurd area, return true if one has been found, false otherwise
 	bool pathfindGuardArea(int teamNumber, bool canSwim, int x, int y, int *dx, int *dy);
+	//! Find the best direction toward clearing area, return true if one has been found, false otherwise
+	bool pathfindClearArea(int teamNumber, bool canSwim, int x, int y, int *dx, int *dy);
 	//! Update the forbidden gradient, 
 	void updateForbiddenGradient(int teamNumber, bool canSwim);
 	template<typename Tint> void updateForbiddenGradient(int teamNumber, bool canSwim);
@@ -580,8 +608,17 @@ public:
 	template<typename Tint> void updateGuardAreasGradient(int teamNumber, bool canSwim);
 	void updateGuardAreasGradient(int teamNumber);
 	void updateGuardAreasGradient();
+	//! Update the clear area gradient
+	void updateClearAreasGradient(int teamNumber, bool canSwim);
+	template<typename Tint> void updateClearAreasGradient(int teamNumber, bool canSwim);
+	void updateClearAreasGradient(int teamNumber);
+	void updateClearAreasGradient();
+	
+	///Implements A* algorithm for point to point pathfinding. Does not cache path, designed to be fast
+	bool pathfindPointToPoint(int x, int y, int targetX, int targetY, int *dx, int *dy, bool canSwim, Uint32 teamMask, int maximumLength);
 	
 	void initExploredArea(int teamNumber);
+	void makeDiscoveredAreasExplored(int teamNumber);
 	void updateExploredArea(int teamNumber);
 	
 protected:
@@ -696,6 +733,9 @@ public:
 	//! true = clear area
 	Utilities::BitArray localClearAreaMap;
 	
+	///This is the maximum fertility of any point on the map
+	Uint16 fertilityMaximum;
+	
 public:
 	// Used to go to ressources
 	//TODO: make the 32 into a variable TEAM_COUNT to save memory in almost all cases
@@ -707,18 +747,37 @@ public:
 	//[int team][bool unitCanSwim]
 	Uint8 *forbiddenGradient[32][2];
 	
-	// Used to attrack idle warriors into guard areas
+	// Used to attract idle warriors into guard areas
 	//[int team][bool unitCanSwim]
 	Uint8 *guardAreasGradient[32][2];
 	
+	// Used to attract idle workers into clearing
+	// areas that aren't clear
+	Uint8 *clearAreasGradient[32][2];
+	
 	// Used to guide explorers
 	//[int team]
 	// 0=unexplored, 255=just explored
 	Uint8 *exploredArea[32];
 	
+	/// This shows how many "claims" there are on a particular ressource square
+	/// This is so that not all 150 free units go after one piece of wood
+	/// Each square is the gid of the claiming unit
+	Uint16 *clearingAreaClaims[32];
+	
+	/// These are integers that tell whether an immobile unit is standing on the
+	/// square, and if so, what team number it is. In terms of the engine, these
+	/// are treated like forbidden areas
+	Uint8 *immobileUnits;
+	
 protected:
 	//Used for scheduling computation time.
 	bool gradientUpdated[32][MAX_NB_RESSOURCES][2];
+	//Used for scheduling computation time on the guard area gradients
+	bool guardGradientUpdated[32][2];
+	//Used for scheduling computation time on the clear area gradients
+	bool clearGradientUpdated[32][2];
+	
 	Uint8 *undermap;
 	Uint8 **listedAddr;
 	size_t size;
@@ -727,6 +786,44 @@ protected:
 	Sint32 wSector, hSector;
 	int sizeSector;
 	
+	
+	///This is a single point in the array used for A* algorithm
+	struct AStarAlgorithmPoint
+	{
+		AStarAlgorithmPoint() : x(-1), y(-1), dx(-1), dy(-1), moveCost(static_cast<Uint16>(-1)), totalCost(static_cast<Uint16>(-1)), isClosed(false) { }
+		AStarAlgorithmPoint(Sint16 x, Sint16 y, Sint16 dx, Sint16 dy, Uint16 moveCost, Uint16 totalCost, bool isClosed) : x(x), y(y), dx(dx), dy(dy), moveCost(moveCost), totalCost(totalCost), isClosed(isClosed) {}
+		//Pos x
+		Sint16 x;
+		//Pos y
+		Sint16 y;
+		//The direction from the starting point that leads to this path
+		Sint16 dx;
+		//The direction from the starting point that leads to this path
+		Sint16 dy;
+		//Cost to get to square x
+		Uint16 moveCost;
+		//Cost to get to square x + estimate to get to the end
+		Uint16 totalCost;
+		//Whether this cell has been examined
+		bool isClosed;
+	};
+	
+	///This is a function-object that compares two points based on their total score in the A* algorithm
+	struct AStarComparator
+	{
+		AStarComparator(const AStarAlgorithmPoint* points) : points(points) {}
+		bool operator()(int lhs, int rhs)
+		{
+			if(points[lhs].totalCost > points[rhs].totalCost)
+				return true;
+			return false;
+		}
+		const AStarAlgorithmPoint* points;
+	};
+	
+	//This array is kept and re-used for every point-to-point pathfind call
+	AStarAlgorithmPoint* astarpoints;
+	std::vector<int> astarExaminedPoints;
 
 public:
 	Uint32 checkSum(bool heavy);
