@@ -18,6 +18,7 @@
 
 #include "MapHeader.h"
 #include "Game.h"
+#include <algorithm>
 
 MapHeader::MapHeader()
 {
@@ -34,6 +35,7 @@ void MapHeader::reset()
 	mapName = "";
 	mapOffset = 0;
 	isSavedGame=false;
+	resetGameSHA1();
 }
 
 
@@ -54,9 +56,18 @@ bool MapHeader::load(GAGCore::InputStream *stream)
 	mapName = stream->readText("mapName");
 	versionMajor = stream->readSint32("versionMajor");
 	versionMinor = stream->readSint32("versionMinor");
+
 	numberOfTeams = stream->readSint32("numberOfTeams");
 	mapOffset = stream->readUint32("mapOffset");
 	isSavedGame = stream->readUint8("isSavedGame");
+	if(versionMinor==67)
+		stream->readUint32("checksum");
+	
+	if(versionMinor>=68)
+	{
+		stream->read(SHA1, 20, "SHA1");
+	}
+	
 	stream->readEnterSection("teams");
 	for(int i=0; i<numberOfTeams; ++i)
 	{
@@ -83,6 +94,7 @@ void MapHeader::save(GAGCore::OutputStream *stream)
 	stream->writeSint32(numberOfTeams, "numberOfTeams");
 	stream->writeUint32(mapOffset, "mapOffset");
 	stream->writeUint8(isSavedGame, "isSavedGame");
+	stream->write(SHA1, 20, "SHA1");
 	stream->writeEnterSection("teams");
 	for(int i=0; i<numberOfTeams; ++i)
 	{
@@ -193,6 +205,30 @@ void MapHeader::setIsSavedGame(bool newIsSavedGame)
 }
 
 
+
+void MapHeader::setGameSHA1(Uint8 SHA1sum[20])
+{
+	for(int i=0; i<20; ++i)
+		SHA1[i] = SHA1sum[i];
+}
+
+
+
+Uint8* MapHeader::getGameSHA1()
+{
+	return SHA1;
+}
+
+
+
+void MapHeader::resetGameSHA1()
+{
+	for(int i=0; i<20; ++i)
+		SHA1[i] = 0;
+}
+
+
+
 Uint32 MapHeader::checkSum() const
 {
 	Sint32 cs = 0;
@@ -207,12 +243,11 @@ Uint32 MapHeader::checkSum() const
 
 bool MapHeader::operator!=(const MapHeader& rhs) const
 {
-	if(rhs.versionMajor != versionMajor ||
-		rhs.versionMinor != versionMinor ||
-		rhs.numberOfTeams != numberOfTeams ||
+	if( rhs.numberOfTeams != numberOfTeams ||
 		rhs.mapOffset != mapOffset ||
 		rhs.isSavedGame != isSavedGame ||
-		rhs.mapName != mapName)
+		rhs.mapName != mapName ||
+		!std::equal(SHA1, SHA1+20, rhs.SHA1))
 		return true;
 	return false;
 }
