@@ -21,7 +21,7 @@
 
 #include "Stream.h"
 #include "Version.h"
-#include "Team.h"
+#include "BaseTeam.h"
 
 ///This is the map header. It is static with the map, and does not change from game to game if
 ///the user is playing on the same map. It holds small details about a map that aren't placed
@@ -38,7 +38,7 @@ public:
 	/// Loads map header information from the stream
 	bool load(GAGCore::InputStream *stream);
 	
-	/// Saves map header information to the stream
+	/// Saves map header information to the stream.
 	void save(GAGCore::OutputStream *stream);
 
 	/// Returns the version major
@@ -85,9 +85,19 @@ public:
 	/// Sets whether or no this header represents a saved game
 	void setIsSavedGame(bool isSavedGame);
 
+	/// Sets the complete game checksum
+	void setGameSHA1(Uint8 SHA1sum[20]);
+	
+	/// Returns the complete game checksum
+	Uint8* getGameSHA1();
+	
+	/// Returns the complete game checksum
+	void resetGameSHA1();
+
 	/// Returns a checksum of the map header information
 	Uint32 checkSum() const;
 	
+	///Comparison does *not* count file version numbers
 	bool operator!=(const MapHeader& rhs) const;
 private:
 	/// Major map version. Changes only with structural modification
@@ -101,12 +111,15 @@ private:
 	/// the complete file.
 	Uint32 mapOffset;
 	
-	///The teams in the map. BaseTeam is used to allow access to information like team numbers and
-	///team colors without loading the entire game.
+	/// The teams in the map. BaseTeam is used to allow access to information like team numbers and
+	/// team colors without loading the entire game.
 	BaseTeam teams[32];
 	
-	///If this is true, this map header represents a saved game, rather than a new map
+	/// If this is true, this map header represents a saved game, rather than a new map
 	bool isSavedGame;
+	
+	/// Set to the complete files SHA1
+	Uint8 SHA1[20];
 
 	std::string mapName;
 };
