@@ -23,6 +23,7 @@
 #include "FormatableString.h"
 #include "Toolkit.h"
 #include "StringTable.h"
+#include "NetMessage.h"
 
 MultiplayerGame::MultiplayerGame(boost::shared_ptr<YOGClient> client)
 	: client(client), gjcState(NothingYet), creationState(YOGCreateRefusalUnknown), joinState(YOGJoinRefusalUnknown), playerManager(gameHeader)
@@ -129,7 +130,7 @@ MultiplayerGame::GameJoinCreationState MultiplayerGame::getGameJoinCreationState
 
 
 
-YOGGameCreateRefusalReason MultiplayerGame::getGameCreationState()
+YOGServerGameCreateRefusalReason MultiplayerGame::getGameCreationState()
 {
 	return creationState;
 }
@@ -137,7 +138,7 @@ YOGGameCreateRefusalReason MultiplayerGame::getGameCreationState()
 
 
 
-YOGGameJoinRefusalReason MultiplayerGame::getGameJoinState()
+YOGServerGameJoinRefusalReason MultiplayerGame::getGameJoinState()
 {
 	return joinState;
 }
@@ -147,10 +148,16 @@ YOGGameJoinRefusalReason MultiplayerGame::getGameJoinState()
 void MultiplayerGame::setMapHeader(MapHeader& nmapHeader)
 {
 	mapHeader = nmapHeader;
-	shared_ptr<NetSendMapHeader> message(new NetSendMapHeader(mapHeader));
+		
+	NetReteamingInformation info = constructReteamingInformation(mapHeader.getFileName());
+	shared_ptr<NetSendReteamingInformation> message(new NetSendReteamingInformation(info));
 	client->sendNetMessage(message);
+	
+	shared_ptr<NetSendMapHeader> message2(new NetSendMapHeader(mapHeader));
+	client->sendNetMessage(message2);
 
 	playerManager.setNumberOfTeams(mapHeader.getNumberOfTeams());
+	playerManager.setReteamingInformation(info);
 }
 
 
@@ -287,7 +294,7 @@ void MultiplayerGame::sendMessage(const std::string& message)
 	boost::shared_ptr<YOGMessage> tmessage(new YOGMessage);
 	tmessage->setSender(client->getUsername());
 	tmessage->setMessage(message);
-	tmessage->setMessageType(YOGGameMessage);
+	tmessage->setMessageType(YOGServerGameMessage);
 	client->sendMessage(tmessage);
 }
 
@@ -424,7 +431,7 @@ void MultiplayerGame::recieveMessage(boost::shared_ptr<NetMessage> message)
 			shared_ptr<NetSendOrder> info = static_pointer_cast<NetSendOrder>(message);
 			shared_ptr<Order> order = info->getOrder();
 			if(order->getOrderType() == ORDER_PLAYER_QUIT_GAME)
-				order->gameCheckSum = -1;
+				order->gameCheckSum = static_cast<unsigned int>(-1);
 			netEngine->pushOrder(order, order->sender, false);
 		}
 	}
@@ -474,6 +481,7 @@ void MultiplayerGame::recieveMessage(boost::shared_ptr<NetMessage> message)
 	{
 		shared_ptr<NetSetLatencyMode> info = static_pointer_cast<NetSetLatencyMode>(message);
 		gameHeader.setGameLatency(info->getLatencyAdjustment());
+		//std::cout<<"info->getLatencyAdjustment()="<<(int)(info->getLatencyAdjustment())<<std::endl;
 	}
 	if(type==MNetPlayerJoinsGame)
 	{
@@ -507,6 +515,11 @@ void MultiplayerGame::recieveMessage(boost::shared_ptr<NetMessage> message)
 		shared_ptr<MGPlayerListChangedEvent> event(new MGPlayerListChangedEvent);
 		sendToListeners(event);
 	}
+	if(type==MNetSendReteamingInformation)
+	{
+		shared_ptr<NetSendReteamingInformation> info = static_pointer_cast<NetSendReteamingInformation>(message);
+		playerManager.setReteamingInformation(info->getReteamingInfo());
+	}
 }
 
 
@@ -560,6 +573,22 @@ void MultiplayerGame::sendToListeners(boost::shared_ptr<MultiplayerGameEvent> ev
 
 
 
+NetReteamingInformation MultiplayerGame::constructReteamingInformation(const std::string& file)
+{
+	NetReteamingInformation info;
+	GameHeader game = Engine::loadGameHeader(file);
+	for(int i=0; i<32; ++i)
+	{
+		if(game.getBasePlayer(i).type == Player::P_IP)
+		{
+			info.setPlayerToTeam(game.getBasePlayer(i).name, game.getBasePlayer(i).teamNumber);
+		}
+	}
+	return info;
+}
+
+
+
 int MultiplayerGame::getLocalPlayer()
 {
 	for(int i=0; i<gameHeader.getNumberOfPlayers(); ++i)
@@ -569,6 +598,7 @@ int MultiplayerGame::getLocalPlayer()
 			return gameHeader.getBasePlayer(i).number;
 		}
 	}
+	return -1;
 }
 
 
