@@ -20,7 +20,7 @@
 */
 
 #include "MultiplayerGameScreen.h"
-#include "YOGScreen.h"
+#include "YOGClientLobbyScreen.h"
 #include "Utilities.h"
 #include "GlobalContainer.h"
 #include "NetConsts.h"
@@ -35,9 +35,10 @@
 #include <StringTable.h>
 
 #include "IRC.h"
+#include "YOGMessage.h"
 
 MultiplayerGameScreen::MultiplayerGameScreen(boost::shared_ptr<MultiplayerGame> game, boost::shared_ptr<YOGClient> client, boost::shared_ptr<IRCTextMessageHandler> ircChat)
-	: game(game), gameChat(new YOGChatChannel(-1, client, this)), ircChat(ircChat)
+	: game(game), gameChat(new YOGClientChatChannel(static_cast<unsigned int>(-1), client)), ircChat(ircChat)
 {
 	// we don't want to add AI_NONE
 	for (size_t i=1; i<AI::SIZE; i++)
@@ -110,11 +111,13 @@ MultiplayerGameScreen::MultiplayerGameScreen(boost::shared_ptr<MultiplayerGame>
 	updateJoinedPlayers();
 	
 	game->addEventListener(this);
+	gameChat->addListener(this);
 }
 
 MultiplayerGameScreen::~MultiplayerGameScreen()
 {
 	game->removeEventListener(this);
+	gameChat->removeListener(this);
 }
 
 void MultiplayerGameScreen::onTimer(Uint32 tick)
@@ -157,16 +160,12 @@ void MultiplayerGameScreen::onAction(Widget *source, Action action, int par1, in
 	}
 	else if (action==TEXT_VALIDATED)
 	{
-//		game->sendMessage(textInput->getText());
-//		boost::shared_ptr<IRC> irc = textMessage->getIRC();
-//		if(irc)
-//		{
-//			irc->sendCommand(textInput->getText());
-//		}
-		boost::shared_ptr<YOGMessage> message(new YOGMessage(textInput->getText(), game->getUsername(), YOGNormalMessage));
-		gameChat->sendMessage(message);
-		textInput->setText("");
-		
+		if(textInput->getText() != "")
+		{
+			boost::shared_ptr<YOGMessage> message(new YOGMessage(textInput->getText(), game->getUsername(), YOGNormalMessage));
+			gameChat->sendMessage(message);
+			textInput->setText("");
+		}
 	}
 }
 
