@@ -23,13 +23,15 @@
 #include "Stream.h"
 #include "YOGConsts.h"
 #include "YOGGameInfo.h"
-#include "YOGPlayerInfo.h"
+#include "YOGPlayerSessionInfo.h"
 #include "YOGMessage.h"
 #include <string>
 #include <vector>
 #include <boost/shared_ptr.hpp>
 #include "MapHeader.h"
 #include "GameHeader.h"
+#include "Player.h"
+#include "NetReteamingInformation.h"
 
 
 using namespace boost;
@@ -37,38 +39,52 @@ using namespace boost;
 ///This is the enum of message types
 enum NetMessageType
 {
+	///These must be kept in this order to maintain compatibility with future versions of glob2
 	MNetAcceptRegistration,
-	MNetAttemptJoinGame,
 	MNetAttemptLogin,
 	MNetAttemptRegistration,
+	MNetDisconnect,
+	MNetLoginSuccessful,
+	MNetPing,
+	MNetPingReply,
+	MNetRefuseLogin,
+	MNetRefuseRegistration,
+	MNetSendClientInformation,
+	MNetSendServerInformation,
+	
+	//These are all glob2 version dependent and can be kept in any order	
+	MNetAddAI,
+	MNetAttemptJoinGame,
+	MNetChangePlayersTeam,
 	MNetCreateGame,
 	MNetCreateGameAccepted,
 	MNetCreateGameRefused,
-	MNetDisconnect,
+	MNetEveryoneReadyToLaunch,
 	MNetGameJoinAccepted,
 	MNetGameJoinRefused,
 	MNetKickPlayer,
 	MNetLeaveGame,
-	MNetLoginSuccessful,
+	MNetNotEveryoneReadyToLaunch,
 	MNetNotReadyToLaunch,
 	MNetPlayerJoinsGame,
-	MNetPlayerLeavesGame,
 	MNetReadyToLaunch,
-	MNetRefuseLogin,
-	MNetRefuseRegistration,
+	MNetRefuseGameStart,
+	MNetRemoveAI,
+	MNetRequestGameStart,
 	MNetRequestMap,
 	MNetRequestNextChunk,
-	MNetSendClientInformation,
 	MNetSendFileChunk,
 	MNetSendFileInformation,
 	MNetSendGameHeader,
+	MNetSendGamePlayerInfo,
 	MNetSendMapHeader,
 	MNetSendOrder,
-	MNetSendServerInformation,
 	MNetSendYOGMessage,
+	MNetSetLatencyMode,
 	MNetStartGame,
 	MNetUpdateGameList,
 	MNetUpdatePlayerList,
+	MNetSendReteamingInformation,
 	//type_append_marker
 };
 
@@ -125,11 +141,14 @@ public:
 	
 	///Changes the Order that NetSendOrder holds. This will
 	///delete an Order that was already present.
-	void changeOrder(boost::shared_ptr<Order> newOrder);
+	void addOrder(boost::shared_ptr<Order> newOrder);
 	
-	///Returns the Order that NetSendOrder holds.
+	///Returns the Order that NetSendOrder holds
 	boost::shared_ptr<Order> getOrder();
 
+	///Changes the Order that NetSendOrder holds
+	void changeOrder(boost::shared_ptr<Order> newOrder);
+
 	///Returns MNetSendOrder
 	Uint8 getMessageType() const;
 
@@ -505,7 +524,7 @@ public:
 	NetUpdatePlayerList();
 
 	///This computes the differences between the two lists of players. These can be of any container,
-	///so long as they store YOGPlayerInfo
+	///so long as they store YOGPlayerSessionInfo
 	template<typename container> void updateDifferences(const container& original, const container& updated);
 
 	///Returns MNetUpdatePlayerList
@@ -529,7 +548,7 @@ public:
 
 private:
 	std::vector<Uint16> removedPlayers;
-	std::vector<YOGPlayerInfo> updatedPlayers;
+	std::vector<YOGPlayerSessionInfo> updatedPlayers;
 };
 
 
@@ -607,13 +626,16 @@ private:
 
 
 ///NetGameJoinAccepted means that a NetAttemptJoinGame was accepted and the player is now
-///joined in the game 
+///joined in the game. It comes with some information about the joined game
 class NetGameJoinAccepted : public NetMessage
 {
 public:
 	///Creates a NetGameJoinAccepted message
 	NetGameJoinAccepted();
 
+	///Creates a NetGameJoinAccepted message with the chat channel of the joined game
+	NetGameJoinAccepted(Uint32 chatChannel);
+
 	///Returns MNetGameJoinAccepted
 	Uint8 getMessageType() const;
 
@@ -629,6 +651,11 @@ public:
 
 	///Compares with another NetGameJoinAccepted
 	bool operator==(const NetMessage& rhs) const;
+
+	///Returns the chat channel of the joined game
+	Uint32 getChatChannel() const;
+private:
+	Uint32 chatChannel;
 };
 
 
@@ -639,7 +666,7 @@ class NetGameJoinRefused : public NetMessage
 {
 public:
 	///Creates a NetGameJoinRefused message
-	NetGameJoinRefused(YOGGameJoinRefusalReason reason);
+	NetGameJoinRefused(YOGServerGameJoinRefusalReason reason);
 
 	///Creates a NetGameJoinRefused message
 	NetGameJoinRefused();
@@ -661,9 +688,9 @@ public:
 	bool operator==(const NetMessage& rhs) const;
 	
 	///Returns the reason why the player could not join the game.
-	YOGGameJoinRefusalReason getRefusalReason() const;
+	YOGServerGameJoinRefusalReason getRefusalReason() const;
 private:
-	YOGGameJoinRefusalReason reason;
+	YOGServerGameJoinRefusalReason reason;
 };
 
 
@@ -674,7 +701,7 @@ class NetSendYOGMessage : public NetMessage
 {
 public:
 	///Creates a NetSendYOGMessage message
-	NetSendYOGMessage(boost::shared_ptr<YOGMessage> message);
+	NetSendYOGMessage(Uint32 channel, boost::shared_ptr<YOGMessage> message);
 
 	///Creates a NetSendYOGMessage message
 	NetSendYOGMessage();
@@ -694,10 +721,14 @@ public:
 
 	///Compares with another NetSendYOGMessage
 	bool operator==(const NetMessage& rhs) const;
+
+	///Returns the channel
+	Uint32 getChannel() const;
 	
 	///Returns the YOG message
 	boost::shared_ptr<YOGMessage> getMessage() const;
 private:
+	Uint32 channel;
 	boost::shared_ptr<YOGMessage> message;
 };
 
@@ -739,13 +770,16 @@ private:
 
 
 
-///NetCreateGameAccepted
+///Tells the player that their game creation was accepted, along with a bit of information about the newly created game
 class NetCreateGameAccepted : public NetMessage
 {
 public:
 	///Creates a NetCreateGameAccepted message
 	NetCreateGameAccepted();
 
+	///Creates a NetCreateGameAccepted message with the chat channel for the new game
+	NetCreateGameAccepted(Uint32 chatChannel);
+
 	///Returns MNetCreateGameAccepted
 	Uint8 getMessageType() const;
 
@@ -760,7 +794,13 @@ public:
 	std::string format() const;
 
 	///Compares with another NetCreateGameAccepted
-	bool operator==(const NetMessage& rhs) const;
+	bool operator==(const NetMessage& rhs) const;	
+
+	///Retrieves the chat channel for the new game
+	Uint32 getChatChannel() const;
+
+private:
+	Uint32 chatChannel;
 };
 
 
@@ -774,7 +814,7 @@ public:
 	NetCreateGameRefused();
 
 	///Creates a NetCreateGameRefused message
-	NetCreateGameRefused(YOGGameCreateRefusalReason reason);
+	NetCreateGameRefused(YOGServerGameCreateRefusalReason reason);
 
 	///Returns MNetCreateGameRefused
 	Uint8 getMessageType() const;
@@ -793,15 +833,16 @@ public:
 	bool operator==(const NetMessage& rhs) const;
 	
 	///Returns the reason why the player could not join the game.
-	YOGGameCreateRefusalReason getRefusalReason() const;
+	YOGServerGameCreateRefusalReason getRefusalReason() const;
 private:
-	YOGGameCreateRefusalReason reason;
+	YOGServerGameCreateRefusalReason reason;
 };
 
 
 
 
-///NetSendGameHeader
+///NetSendGameHeader, sends the game header, but without any player information. Player information is sent in
+///NetSendGamePlayerInfo
 class NetSendGameHeader : public NetMessage
 {
 public:
@@ -827,8 +868,8 @@ public:
 	///Compares with another NetSendGameHeader
 	bool operator==(const NetMessage& rhs) const;
 	
-	///Returns the game header
-	const GameHeader& getGameHeader() const;
+	///Downloads information into the given game header
+	void downloadToGameHeader(GameHeader& header);
 private:
 	GameHeader gameHeader;
 };
@@ -836,17 +877,17 @@ private:
 
 
 
-///NetPlayerJoinsGame
-class NetPlayerJoinsGame : public NetMessage
+///NetSendGamePlayerInfo. This sends the BasePlayer portion of GameHeader
+class NetSendGamePlayerInfo : public NetMessage
 {
 public:
-	///Creates a NetPlayerJoinsGame message
-	NetPlayerJoinsGame();
+	///Creates a NetSendGamePlayerInfo message
+	NetSendGamePlayerInfo();
 
-	///Creates a NetPlayerJoinsGame message
-	NetPlayerJoinsGame(Uint16 playerID);
+	///Creates a NetSendGamePlayerInfo message.
+	NetSendGamePlayerInfo(GameHeader& header);
 
-	///Returns MNetPlayerJoinsGame
+	///Returns MNetSendGamePlayerInfo
 	Uint8 getMessageType() const;
 
 	///Encodes the data
@@ -855,52 +896,18 @@ public:
 	///Decodes the data
 	void decodeData(GAGCore::InputStream* stream);
 
-	///Formats the NetPlayerJoinsGame message with a small amount
+	///Formats the NetSendGamePlayerInfo message with a small amount
 	///of information.
 	std::string format() const;
 
-	///Compares with another NetPlayerJoinsGame
+	///Compares with another NetSendGamePlayerInfo
 	bool operator==(const NetMessage& rhs) const;
-	
-	///Returns the player ID
-	Uint16 getPlayerID() const;
+
+	///Downloads all of the player info to this game header
+	void downloadToGameHeader(GameHeader& header);
 private:
-	Uint16 playerID;
-};
+	GameHeader gameHeader;
 
-
-
-
-///NetPlayerLeavesGame
-class NetPlayerLeavesGame : public NetMessage
-{
-public:
-	///Creates a NetPlayerLeavesGame message
-	NetPlayerLeavesGame();
-	
-	///Creates a NetPlayerLeavesGame message
-	NetPlayerLeavesGame(Uint16 playerID);
-
-	///Returns MNetPlayerLeavesGame
-	Uint8 getMessageType() const;
-
-	///Encodes the data
-	void encodeData(GAGCore::OutputStream* stream) const;
-
-	///Decodes the data
-	void decodeData(GAGCore::InputStream* stream);
-
-	///Formats the NetPlayerLeavesGame message with a small amount
-	///of information.
-	std::string format() const;
-
-	///Compares with another NetPlayerLeavesGame
-	bool operator==(const NetMessage& rhs) const;
-	
-	///Returns the player ID
-	Uint16 getPlayerID() const;
-private:
-	Uint16 playerID;
 };
 
 
@@ -1029,7 +1036,7 @@ public:
 	Uint32 getChunkSize() const;
 private:
 	Uint32 size;
-	Uint8 data[1024];
+	Uint8 data[4096];
 };
 
 
@@ -1198,6 +1205,401 @@ private:
 
 
 
+
+///NetEveryoneReadyToLaunch
+class NetEveryoneReadyToLaunch : public NetMessage
+{
+public:
+	///Creates a NetEveryoneReadyToLaunch message
+	NetEveryoneReadyToLaunch();
+
+	///Returns MNetEveryoneReadyToLaunch
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetEveryoneReadyToLaunch message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetEveryoneReadyToLaunch
+	bool operator==(const NetMessage& rhs) const;
+};
+
+
+
+
+///NetNotEveryoneReadyToLaunch
+class NetNotEveryoneReadyToLaunch : public NetMessage
+{
+public:
+	///Creates a NetNotEveryoneReadyToLaunch message
+	NetNotEveryoneReadyToLaunch();
+
+	///Returns MNetNotEveryoneReadyToLaunch
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetNotEveryoneReadyToLaunch message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetNotEveryoneReadyToLaunch
+	bool operator==(const NetMessage& rhs) const;
+};
+
+
+
+
+///NetRemoveAI
+class NetRemoveAI : public NetMessage
+{
+public:
+	///Creates a NetRemoveAI message
+	NetRemoveAI();
+
+	///Creates a NetRemoveAI message
+	NetRemoveAI(Uint8 playerNum);
+
+	///Returns MNetRemoveAI
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetRemoveAI message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetRemoveAI
+	bool operator==(const NetMessage& rhs) const;
+
+	///Retrieves playerNum
+	Uint8 getPlayerNumber() const;
+private:
+private:
+	Uint8 playerNum;
+};
+
+
+
+
+///NetChangePlayersTeam
+class NetChangePlayersTeam : public NetMessage
+{
+public:
+	///Creates a NetChangePlayersTeam message
+	NetChangePlayersTeam();
+
+	///Creates a NetChangePlayersTeam message
+	NetChangePlayersTeam(Uint8 player, Uint8 team);
+
+	///Returns MNetChangePlayersTeam
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetChangePlayersTeam message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetChangePlayersTeam
+	bool operator==(const NetMessage& rhs) const;
+
+	///Retrieves player
+	Uint8 getPlayer() const;
+
+	///Retrieves team
+	Uint8 getTeam() const;
+private:
+private:
+	Uint8 player;
+	Uint8 team;
+};
+
+
+
+
+///NetRequestGameStart
+class NetRequestGameStart : public NetMessage
+{
+public:
+	///Creates a NetRequestGameStart message
+	NetRequestGameStart();
+
+	///Returns MNetRequestGameStart
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetRequestGameStart message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetRequestGameStart
+	bool operator==(const NetMessage& rhs) const;
+};
+
+
+
+
+///NetRefuseGameStart
+class NetRefuseGameStart : public NetMessage
+{
+public:
+	///Creates a NetRefuseGameStart message
+	NetRefuseGameStart();
+
+	///Creates a NetRefuseGameStart message
+	NetRefuseGameStart(YOGServerGameStartRefusalReason refusalReason);
+
+	///Returns MNetRefuseGameStart
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetRefuseGameStart message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetRefuseGameStart
+	bool operator==(const NetMessage& rhs) const;
+
+	///Retrieves refusalReason
+	YOGServerGameStartRefusalReason getRefusalReason() const;
+private:
+private:
+	YOGServerGameStartRefusalReason refusalReason;
+};
+
+
+
+
+///NetPing
+class NetPing : public NetMessage
+{
+public:
+	///Creates a NetPing message
+	NetPing();
+
+	///Returns MNetPing
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetPing message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetPing
+	bool operator==(const NetMessage& rhs) const;
+};
+
+
+
+
+///NetPingReply
+class NetPingReply : public NetMessage
+{
+public:
+	///Creates a NetPingReply message
+	NetPingReply();
+
+	///Returns MNetPingReply
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetPingReply message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetPingReply
+	bool operator==(const NetMessage& rhs) const;
+};
+
+
+
+
+///NetSetLatencyMode
+class NetSetLatencyMode : public NetMessage
+{
+public:
+	///Creates a NetSetLatencyMode message
+	NetSetLatencyMode();
+
+	///Creates a NetSetLatencyMode message
+	NetSetLatencyMode(Uint8 latencyAdjustment);
+
+	///Returns MNetSetLatencyMode
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetSetLatencyMode message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetSetLatencyMode
+	bool operator==(const NetMessage& rhs) const;
+
+	///Retrieves latencyAdjustment
+	Uint8 getLatencyAdjustment() const;
+private:
+private:
+	Uint8 latencyAdjustment;
+};
+
+
+
+
+///NetPlayerJoinsGame
+class NetPlayerJoinsGame : public NetMessage
+{
+public:
+	///Creates a NetPlayerJoinsGame message
+	NetPlayerJoinsGame();
+
+	///Creates a NetPlayerJoinsGame message
+	NetPlayerJoinsGame(Uint16 playerID, std::string playerName);
+
+	///Returns MNetPlayerJoinsGame
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetPlayerJoinsGame message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetPlayerJoinsGame
+	bool operator==(const NetMessage& rhs) const;
+
+	///Retrieves playerID
+	Uint16 getPlayerID() const;
+
+	///Retrieves playerName
+	std::string getPlayerName() const;
+private:
+private:
+	Uint16 playerID;
+	std::string playerName;
+};
+
+
+
+
+///NetAddAI
+class NetAddAI : public NetMessage
+{
+public:
+	///Creates a NetAddAI message
+	NetAddAI();
+
+	///Creates a NetAddAI message
+	NetAddAI(Uint8 type);
+
+	///Returns MNetAddAI
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetAddAI message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetAddAI
+	bool operator==(const NetMessage& rhs) const;
+
+	///Retrieves type
+	Uint8 getType() const;
+private:
+private:
+	Uint8 type;
+};
+
+
+
+
+///NetSendReteamingInformation
+class NetSendReteamingInformation : public NetMessage
+{
+public:
+	///Creates a NetSendReteamingInformation message
+	NetSendReteamingInformation();
+
+	///Creates a NetSendReteamingInformation message
+	NetSendReteamingInformation(NetReteamingInformation reteamingInfo);
+
+	///Returns MNetSendReteamingInformation
+	Uint8 getMessageType() const;
+
+	///Encodes the data
+	void encodeData(GAGCore::OutputStream* stream) const;
+
+	///Decodes the data
+	void decodeData(GAGCore::InputStream* stream);
+
+	///Formats the NetSendReteamingInformation message with a small amount
+	///of information.
+	std::string format() const;
+
+	///Compares with another NetSendReteamingInformation
+	bool operator==(const NetMessage& rhs) const;
+
+	///Retrieves reteamingInfo
+	NetReteamingInformation getReteamingInfo() const;
+private:
+private:
+	NetReteamingInformation reteamingInfo;
+};
+
+
+
 //message_append_marker
 
 #include <iostream>
@@ -1358,7 +1760,7 @@ template<typename container> void NetUpdatePlayerList::applyDifferences(containe
 	}
 	
 	//Change and/or add the players that are updated
-	for(std::vector<YOGPlayerInfo>::const_iterator i=updatedPlayers.begin(); i!=updatedPlayers.end(); ++i)
+	for(std::vector<YOGPlayerSessionInfo>::const_iterator i=updatedPlayers.begin(); i!=updatedPlayers.end(); ++i)
 	{
 		bool found=false;
 		for(typename container::iterator j=original.begin(); j!=original.end(); ++j)
