@@ -27,7 +27,7 @@
 Order::Order(void)
 {
 	sender=-1;
-	gameCheckSum=-1;
+	gameCheckSum=static_cast<unsigned int>(-1);
 }
 
 boost::shared_ptr<Order> Order::getOrder(const Uint8 *netData, int netDataLength)
@@ -598,13 +598,13 @@ OrderAlterateArea::OrderAlterateArea(const Uint8 *data, int dataLength)
 	assert(good);
 }
 
-OrderAlterateArea::OrderAlterateArea(Uint8 teamNumber, Uint8 type, BrushAccumulator *acc)
+OrderAlterateArea::OrderAlterateArea(Uint8 teamNumber, Uint8 type, BrushAccumulator *acc, const Map* map)
 {
 	assert(acc);
 	_data = NULL;
 	
 	BrushAccumulator::AreaDimensions dim;
-	acc->getBitmap(&mask, &dim);
+	acc->getBitmap(&mask, &dim, map);
 	this->teamNumber = teamNumber;
 	this->type = type;
 	centerX = dim.centerX;
