@@ -141,77 +141,157 @@ SettingsScreen::SettingsScreen()
 	// Audio part
 	audio=new Text(230, 330, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[audio]"), 300);
 	addWidget(audio);
-	audioMute=new OnOffButton(230, 360, 20, 20, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, globalContainer->settings.mute, MUTE);
+	audioMute=new OnOffButton(230, 365, 20, 20, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, globalContainer->settings.mute, MUTE);
 	addWidget(audioMute);
-	audioMuteText=new Text(260, 360, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[mute]"), 200);
+	audioMuteText=new Text(260, 365, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[mute]"), 200);
 	addWidget(audioMuteText);
-	musicVol=new Selector(320, 360, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 180, globalContainer->settings.musicVolume, 256, true);
+	musicVol=new Selector(320, 350, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 180, globalContainer->settings.musicVolume, 256, true);
 	addWidget(musicVol);
+	voiceVol=new Selector(320, 385, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 180, globalContainer->settings.voiceVolume, 256, true);
+	addWidget(voiceVol);
 	musicVolText=new Text(320, 330, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[Music volume]"), 300);
 	addWidget(musicVolText);
+	voiceVolText=new Text(320, 365, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[Voice volume]"), 300);
+	addWidget(voiceVolText);
 	setVisibilityFromAudioSettings();
 	
-	int row=0;
-	int current_column_x=20;
-	int widest_element=0;
+	
+	int first_group_row=0;
+	int first_group_current_column_x=20;
+	int first_group_widest_element=0;
+	
+	int second_group_row=1;
+	int second_group_first_row_x=170;
+	int second_group_current_column_x=170;
+	int second_group_widest_element=0;
+	
 	for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
 	{
-		std::string name=IntBuildingType::typeFromShortNumber(t);
 		for(int l=0; l<6; ++l)
 		{
-			if(globalContainer->buildingsTypes.getByType(name, l/2, (l+1)%2) != NULL && globalContainer->settings.defaultUnitsAssigned[t][l]>0)
+			unitRatios[t][l]=NULL;
+			unitRatioTexts[t][l]=NULL;
+		}
+	}
+	
+	///First group are completed buildings, Inn, Swarm and Defence tower
+	for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
+	{
+		if(t==IntBuildingType::EXPLORATION_FLAG || t==IntBuildingType::WAR_FLAG || t==IntBuildingType::CLEARING_FLAG)
+			continue;
+		std::string name=IntBuildingType::typeFromShortNumber(t);
+		for(int l=0; l<3; ++l)
+		{
+			if(globalContainer->buildingsTypes.getByType(name, l, false) != NULL && globalContainer->settings.defaultUnitsAssigned[t][l*2+1]>0)
 			{
-				unitRatios[t][l] = new Number(current_column_x, 70+45*row, 100, 18, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 20, "menu");
-				addNumbersFor(0, 20, unitRatios[t][l]);
-				unitRatios[t][l]->setNth(globalContainer->settings.defaultUnitsAssigned[t][l]);
-				unitRatios[t][l]->visible=false;
-				addWidget(unitRatios[t][l]);
-
-				widest_element = std::max(widest_element, unitRatios[t][l]->getWidth());
-
-				std::string keyname="[";
-				if((l+1)%2)
-					keyname+="build ";
-				keyname+=name + " level " + boost::lexical_cast<std::string>(l/2) + "]";
-				unitRatioTexts[t][l]=new Text(current_column_x, 50+45*row, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString(keyname.c_str()));
-				addWidget(unitRatioTexts[t][l]);
-				unitRatioTexts[t][l]->visible=false;
-
-				widest_element = std::max(widest_element, unitRatioTexts[t][l]->getWidth());
-
-				row+=1;
-				if(row==8)
+				int size = addDefaultUnitAssignmentWidget(t, l*2+1, first_group_current_column_x, 100 + 40*first_group_row, 1);
+				first_group_widest_element = std::max(first_group_widest_element, size);
+				
+				first_group_row += 1;
+				if(first_group_row == 8)
 				{
-					row=0;
-					current_column_x+= widest_element+10;
+					first_group_row = 0;
+					first_group_current_column_x += first_group_widest_element;
+					first_group_widest_element = 0;
 				}
 			}
-			else
+		}	
+	}
+	
+	///Second group, at the top is Swarm, Wall, Market, the rest follow horizontally
+	for(int l=0; l<3; ++l)
+	{
+		for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
+		{
+			std::string name=IntBuildingType::typeFromShortNumber(t);
+			//Even numbers represent under-construction, whereas odd numbers represent completed buildings		
+			if(globalContainer->buildingsTypes.getByType(name, l, true) != NULL && globalContainer->settings.defaultUnitsAssigned[t][l*2]>0)
 			{
-				unitRatios[t][l]=NULL;
-				unitRatioTexts[t][l]=NULL;
+				int this_row = second_group_row;
+				int this_x = second_group_current_column_x;
+				if(t == IntBuildingType::SWARM_BUILDING || t == IntBuildingType::STONE_WALL || t == IntBuildingType::MARKET_BUILDING)
+				{
+					this_row = 0;
+					this_x = second_group_first_row_x;
+					second_group_first_row_x += 180;
+				}
+				else
+				{
+					second_group_row += 1;
+				}
+				
+				int size = addDefaultUnitAssignmentWidget(t, l*2, this_x, 100 + 40*this_row, 1);
+				second_group_widest_element = std::max(second_group_widest_element, size);
+				
+				if(second_group_row == 8)
+				{
+					second_group_row = 1;
+					second_group_current_column_x += 180;
+					second_group_widest_element = 0;
+				}
 			}
 		}
 	}
+	
+	first_group_row=0;
+	first_group_current_column_x=20;
+	first_group_widest_element=0;
+	///On the second screen, the only group is first
+	for(int t=IntBuildingType::EXPLORATION_FLAG; t<=IntBuildingType::CLEARING_FLAG; ++t)
+	{
+		int size = addDefaultUnitAssignmentWidget(t, 1, first_group_current_column_x, 100 + 40*first_group_row, 2, true);
+		first_group_widest_element = std::max(first_group_widest_element, size);
+		
+		first_group_row += 1;
+		if(first_group_row == 8)
+		{
+			first_group_row = 0;
+			first_group_current_column_x += first_group_widest_element;
+			first_group_widest_element = 0;
+		}
+	}
+	
+	flags = new TextButton( 10, 60, 120, 20, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[Building Settings]"), BUILDINGSETTINGS);
+	buildings = new TextButton( 140, 60, 120, 20, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[Flag Settings]"), FLAGSETTINGS);
+	
+	unitSettingsExplanation = new Text( 270, 60, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[unit settings explanation]"));
+	unitSettingsHeading1 = new Text( 160, 80, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[construction and upgrades]"));
+	unitSettingsHeading2 = new Text( 10, 80, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[constructed buildings]"));
+
+
+	unitSettingsExplanation->visible = false;
+	unitSettingsHeading1->visible = false;
+	unitSettingsHeading2->visible = false;
+	buildings->visible = false;
+	flags->visible = false;
+
+	addWidget(unitSettingsExplanation);
+	addWidget(unitSettingsHeading1);
+	addWidget(unitSettingsHeading2);
+	addWidget(buildings);
+	addWidget(flags);
 
 	//shortcuts part
-	game_shortcuts=new TextButton( 20, 60, 200, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "menu", Toolkit::getStringTable()->getString("[game shortcuts]"), GAMESHORTCUTS);
+	game_shortcuts=new TextButton( 100, 60, 120, 20, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[game shortcuts]"), GAMESHORTCUTS);
 	game_shortcuts->visible=false;
 
-	editor_shortcuts=new TextButton( 230, 60, 200, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "menu", Toolkit::getStringTable()->getString("[editor shortcuts]"), EDITORSHORTCUTS);
+	editor_shortcuts=new TextButton( 230, 60, 120, 20, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[editor shortcuts]"), EDITORSHORTCUTS);
 	editor_shortcuts->visible=false;
 
-	restore_default_shortcuts = new TextButton(20, 355, 610, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[restore default shortcuts]"), RESTOREDEFAULTSHORTCUTS);
-	restore_default_shortcuts->visible=false;
-
 	shortcut_list = new List(20, 110, 325, 160, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard");
 	action_list = new List(365, 110 , 265, 190, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard");
-	select_key_1 = new KeySelector(20, 275, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", 145, 25);
-	key_2_active = new OnOffButton(170, 275, 25, 25, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, false, SECONDKEY);
-	select_key_2 = new KeySelector(200, 275, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", 145, 25);
-	add_shortcut = new TextButton(20, 305, 300, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[add shortcut]"), ADDSHORTCUT);
-	remove_shortcut = new TextButton(330, 305, 300, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[remove shortcut]"), REMOVESHORTCUT);
-	
+	select_key_1 = new KeySelector(20, 275, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", 100, 25);
+	key_2_active = new OnOffButton(125, 275, 25, 25, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, false, SECONDKEY);
+	select_key_2 = new KeySelector(155, 275, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", 100, 25);
+	pressedUnpressedSelector = new MultiTextButton(260, 275, 80, 25, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", "", PRESSEDSELECTOR);
+	add_shortcut = new TextButton(20, 305, 158, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[add shortcut]"), ADDSHORTCUT);
+	remove_shortcut = new TextButton(188, 305, 157, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[remove shortcut]"), REMOVESHORTCUT);
+	restore_default_shortcuts = new TextButton(365, 305, 265, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[restore default shortcuts]"), RESTOREDEFAULTSHORTCUTS);
+
+	pressedUnpressedSelector->clearTexts();
+	pressedUnpressedSelector->addText(Toolkit::getStringTable()->getString("[on press]"));
+	pressedUnpressedSelector->addText(Toolkit::getStringTable()->getString("[on unpress]"));
+
 	
 	game_shortcuts->visible = false;
 	editor_shortcuts->visible = false;
@@ -221,19 +301,22 @@ SettingsScreen::SettingsScreen()
 	select_key_1->visible = false;
 	key_2_active->visible = false;
 	select_key_2->visible = false;
+	pressedUnpressedSelector->visible = false;
 	add_shortcut->visible = false;
 	remove_shortcut->visible = false;
-	
+	restore_default_shortcuts->visible=false;
+		
 	addWidget(game_shortcuts);
 	addWidget(editor_shortcuts);
-	addWidget(restore_default_shortcuts);
 	addWidget(shortcut_list);
 	addWidget(action_list);
 	addWidget(select_key_1);
 	addWidget(key_2_active);
 	addWidget(select_key_2);
+	addWidget(pressedUnpressedSelector);
 	addWidget(add_shortcut);
 	addWidget(remove_shortcut);
+	addWidget(restore_default_shortcuts);
 
 	currentMode = GameGUIShortcuts;
 
@@ -257,7 +340,7 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 		if (par1==OK)
 		{
 			globalContainer->setUserName(userName->getText());
-			globalContainer->settings.defaultLanguage = Toolkit::getStringTable()->getLang();
+			globalContainer->settings.language = Toolkit::getStringTable()->getStringInLang("[language-code]", Toolkit::getStringTable()->getLang());
 			globalContainer->settings.save();
 			mapeditKeyboardManager.saveKeyboardLayout();
 			guiKeyboardManager.saveKeyboardLayout();
@@ -269,7 +352,10 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 			if (gfxAltered)
 				updateGfxCtx();
 
-			Toolkit::getStringTable()->setLang(globalContainer->settings.defaultLanguage);
+			Toolkit::getStringTable()->setLang(Toolkit::getStringTable()->getLangCode(globalContainer->settings.language));
+
+			///Send the old volume to the mixer
+			globalContainer->mix->setVolume(globalContainer->settings.musicVolume, globalContainer->settings.voiceVolume, globalContainer->settings.mute);
 
 			endExecute(par1);
 		}
@@ -295,11 +381,14 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 			audioMute->visible=true;
 			musicVol->visible=true;
 			musicVolText->visible=true;
+			voiceVol->visible=true;
+			voiceVolText->visible=true;
 			rememberUnitButton->visible=true;
 			rememberUnitText->visible=true;
 			scrollwheel->visible=true;
 			scrollwheelText->visible=true;
 			
+			
 			for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
 			{
 				for(int l=0; l<6; ++l)
@@ -311,12 +400,20 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 					}
 				}
 			}
-
+			
+			
+			unitSettingsExplanation->visible=false;
+			unitSettingsHeading1->visible=false;
+			unitSettingsHeading2->visible=false;
+			buildings->visible=false;
+			flags->visible=false;
+			
 			game_shortcuts->visible=false;
 			editor_shortcuts->visible=false;
 			select_key_1->visible=false;
 			key_2_active->visible=false;
 			select_key_2->visible=false;
+			pressedUnpressedSelector->visible=false;
 			shortcut_list->visible=false;
 			action_list->visible=false;
 			add_shortcut->visible=false;
@@ -348,28 +445,27 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 			audioMute->visible=false;
 			musicVol->visible=false;
 			musicVolText->visible=false;
+			voiceVol->visible=false;
+			voiceVolText->visible=false;
 			rememberUnitButton->visible=false;
 			rememberUnitText->visible=false;
 			scrollwheel->visible=false;
 			scrollwheelText->visible=false;
-
-			for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
-			{
-				for(int l=0; l<6; ++l)
-				{
-					if(unitRatios[t][l])
-					{
-						unitRatios[t][l]->visible=true;
-						unitRatioTexts[t][l]->visible=true;
-					}
-				}
-			}
+			
+			activateDefaultAssignedGroupNumber(1);
+			
+			unitSettingsExplanation->visible=true;
+			unitSettingsHeading1->visible=true;
+			unitSettingsHeading2->visible=true;
+			buildings->visible=true;
+			flags->visible=true;
 	
 			game_shortcuts->visible=false;
 			editor_shortcuts->visible=false;
 			select_key_1->visible=false;
 			key_2_active->visible=false;
 			select_key_2->visible=false;
+			pressedUnpressedSelector->visible=false;
 			shortcut_list->visible=false;
 			action_list->visible=false;
 			add_shortcut->visible=false;
@@ -399,11 +495,14 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 			audioMute->visible=false;
 			musicVol->visible=false;
 			musicVolText->visible=false;
+			voiceVol->visible=false;
+			voiceVolText->visible=false;
 			rememberUnitButton->visible=false;
 			rememberUnitText->visible=false;
 			scrollwheel->visible=false;
 			scrollwheelText->visible=false;
 			
+			
 			for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
 			{
 				for(int l=0; l<6; ++l)
@@ -415,12 +514,20 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 					}
 				}
 			}
+			
+			
+			unitSettingsExplanation->visible=false;
+			unitSettingsHeading1->visible=false;
+			unitSettingsHeading2->visible=false;
+			buildings->visible=false;
+			flags->visible=false;
 	
 			game_shortcuts->visible=true;
 			editor_shortcuts->visible=true;
 			select_key_1->visible=true;
 			key_2_active->visible=true;
 			select_key_2->visible=true;
+			pressedUnpressedSelector->visible=true;
 			shortcut_list->visible=true;
 			action_list->visible=true;
 			add_shortcut->visible=true;
@@ -461,6 +568,9 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 			updateActionList();
 			updateShortcutInfoFromSelection();
 		}
+		else if(par1==PRESSEDSELECTOR)
+		{
+		}
 		else if(par1==ADDSHORTCUT)
 		{
 			addNewShortcut();
@@ -469,6 +579,18 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 		{
 			removeShortcut();
 		}
+		else if(par1==BUILDINGSETTINGS)
+		{
+			activateDefaultAssignedGroupNumber(1);
+			unitSettingsHeading1->visible=true;
+			unitSettingsHeading2->visible=true;
+		}
+		else if(par1==FLAGSETTINGS)
+		{
+			activateDefaultAssignedGroupNumber(2);
+			unitSettingsHeading1->visible=false;
+			unitSettingsHeading2->visible=false;
+		}
 	}
 	else if (action==NUMBER_ELEMENT_SELECTED)
 	{
@@ -508,10 +630,31 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 			lowqualityText->setText(Toolkit::getStringTable()->getString("[lowquality]"));
 			customcurText->setText(Toolkit::getStringTable()->getString("[customcur]"));
 
+
+			rememberUnitText->setText(Toolkit::getStringTable()->getString("[remember unit]"));
+			scrollwheelText->setText(Toolkit::getStringTable()->getString("[scroll wheel enabled]"));
+
 			musicVolText->setText(Toolkit::getStringTable()->getString("[Music volume]"));
 			audioMuteText->setText(Toolkit::getStringTable()->getString("[mute]"));
 			
 			rebootWarning->setText(Toolkit::getStringTable()->getString("[Warning, you need to reboot the game for changes to take effect]"));
+			
+			unitSettingsExplanation->setText(Toolkit::getStringTable()->getString("[unit settings explanation]"));
+			unitSettingsHeading1->setText(Toolkit::getStringTable()->getString("[construction and upgrades]"));
+			unitSettingsHeading2->setText(Toolkit::getStringTable()->getString("[constructed buildings]"));
+			buildings->setText(Toolkit::getStringTable()->getString("[Building Settings]"));
+			flags->setText(Toolkit::getStringTable()->getString("[Flag Settings]"));
+			setLanguageTextsForDefaultAssignmentWidgets();
+			
+			game_shortcuts->setText(Toolkit::getStringTable()->getString("[game shortcuts]"));
+			editor_shortcuts->setText(Toolkit::getStringTable()->getString("[editor shortcuts]"));
+			restore_default_shortcuts->setText(Toolkit::getStringTable()->getString("[restore default shortcuts]"));
+			add_shortcut->setText(Toolkit::getStringTable()->getString("[add shortcut]"));
+			remove_shortcut->setText(Toolkit::getStringTable()->getString("[remove shortcut]"));
+			
+			pressedUnpressedSelector->clearTexts();
+			pressedUnpressedSelector->addText(Toolkit::getStringTable()->getString("[on press]"));
+			pressedUnpressedSelector->addText(Toolkit::getStringTable()->getString("[on unpress]"));
 		}
 		else if (source==modeList)
 		{
@@ -533,7 +676,8 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 	else if (action==VALUE_CHANGED)
 	{
 		globalContainer->settings.musicVolume = musicVol->getValue();
-		globalContainer->mix->setVolume(globalContainer->settings.musicVolume, globalContainer->settings.mute);
+		globalContainer->settings.voiceVolume = voiceVol->getValue();
+		globalContainer->mix->setVolume(globalContainer->settings.musicVolume, globalContainer->settings.voiceVolume, globalContainer->settings.mute);
 	}
 	else if (action==BUTTON_STATE_CHANGED)
 	{
@@ -590,7 +734,7 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 		else if (source==audioMute)
 		{
 			globalContainer->settings.mute = audioMute->getState();
-			globalContainer->mix->setVolume(globalContainer->settings.musicVolume, globalContainer->settings.mute);
+		globalContainer->mix->setVolume(globalContainer->settings.musicVolume, globalContainer->settings.voiceVolume, globalContainer->settings.mute);
 			setVisibilityFromAudioSettings();
 		}
 		else if (source==key_2_active)
@@ -623,6 +767,8 @@ void SettingsScreen::setVisibilityFromAudioSettings(void)
 {
 	musicVol->visible = !globalContainer->settings.mute;
 	musicVolText->visible = !globalContainer->settings.mute;
+	voiceVol->visible = !globalContainer->settings.mute;
+	voiceVolText->visible = !globalContainer->settings.mute;
 }
 
 void SettingsScreen::updateGfxCtx(void)
@@ -647,6 +793,91 @@ std::string SettingsScreen::actDisplayModeToString(void)
 
 
 
+int SettingsScreen::addDefaultUnitAssignmentWidget(int type, int level, int x, int y, int group, bool flag)
+{
+	unitRatios[type][level] = new Number(x, y+20, 100, 18, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 20, "menu");
+	addNumbersFor(0, 20, unitRatios[type][level]);
+	unitRatios[type][level]->setNth(globalContainer->settings.defaultUnitsAssigned[type][level]);
+	unitRatios[type][level]->visible=false;
+	addWidget(unitRatios[type][level]);
+
+	std::string text=getDefaultUnitAssignmentText(type, level, flag);
+	unitRatioTexts[type][level]=new Text(x, y, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", text);
+	
+	addWidget(unitRatioTexts[type][level]);
+	unitRatioTexts[type][level]->visible=false;
+	unitRatioGroupNumbers[type][level] = group;
+
+	return std::max(unitRatioTexts[type][level]->getWidth(), unitRatios[type][level]->getWidth());
+}
+
+
+
+std::string SettingsScreen::getDefaultUnitAssignmentText(int type, int level, bool flag)
+{
+	std::string name=IntBuildingType::typeFromShortNumber(type);
+
+	std::string keyname="[";
+	if(flag)
+	{
+		keyname+=name + "]";
+	}
+	else
+	{
+		if((level+1)%2)
+			keyname+="build ";
+		keyname+=name + " level " + boost::lexical_cast<std::string>(level/2) + "]";
+	}
+	return Toolkit::getStringTable()->getString(keyname.c_str());
+}
+
+
+
+void SettingsScreen::setLanguageTextsForDefaultAssignmentWidgets()
+{
+	for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
+	{
+		bool flag = false;
+		if(t == IntBuildingType::EXPLORATION_FLAG || t == IntBuildingType::WAR_FLAG || t == IntBuildingType::CLEARING_FLAG)
+			flag = true;
+		for(int l=0; l<6; ++l)
+		{
+			if(unitRatioTexts[t][l])
+			{
+				unitRatioTexts[t][l]->setText(getDefaultUnitAssignmentText(t, l, flag));
+			}
+		}
+	}
+}
+
+
+
+void SettingsScreen::activateDefaultAssignedGroupNumber(int group)
+{
+	for(int i=0; i<IntBuildingType::NB_BUILDING; ++i)
+	{
+		for(int j=0; j<6; ++j)
+		{
+			if(unitRatioGroupNumbers[i][j] == group)
+			{
+				if(unitRatios[i][j])
+					unitRatios[i][j]->visible=true;
+				if(unitRatioTexts[i][j])
+					unitRatioTexts[i][j]->visible=true;
+			}
+			else
+			{
+				if(unitRatios[i][j])
+					unitRatios[i][j]->visible=false;
+				if(unitRatioTexts[i][j])
+					unitRatioTexts[i][j]->visible=false;
+			}
+		}
+	}
+}
+
+
+
 void SettingsScreen::updateShortcutList(int an)
 {
 	KeyboardManager* m = NULL;
@@ -737,6 +968,11 @@ void SettingsScreen::updateShortcutInfoFromSelection()
 			select_key_2->visible=true;
 		}
 
+		if(i->getKeyPress(0).getPressed())
+			pressedUnpressedSelector->setIndex(0);
+		else
+			pressedUnpressedSelector->setIndex(1);
+
 		action_list->setSelectionIndex(i->getAction());
 		action_list->centerOnItem(action_list->getSelectionIndex());
 	}
@@ -760,9 +996,13 @@ void SettingsScreen::updateKeyboardManagerFromShortcutInfo()
 		std::list<KeyboardShortcut>::iterator i = shortcuts.begin();
 		std::advance(i, selection_n);
 		KeyboardShortcut new_shortcut;
-		new_shortcut.addKeyPress(select_key_1->getKey());
+		
+		KeyPress first = KeyPress(select_key_1->getKey(), (pressedUnpressedSelector->getIndex() == 0 ? true : false));
+		KeyPress second = KeyPress(select_key_2->getKey(), (pressedUnpressedSelector->getIndex() == 0 ? true : false));
+		
+		new_shortcut.addKeyPress(first);
 		if(key_2_active->getState())
-			new_shortcut.addKeyPress(select_key_2->getKey());
+			new_shortcut.addKeyPress(second);
 		new_shortcut.setAction(action_list->getSelectionIndex());
 		(*i) = new_shortcut;
 		updateShortcutList(selection_n);
