@@ -35,6 +35,7 @@ namespace GAGGUI
 	class TextInput;
 	class TextButton;
 	class OnOffButton;
+	class MultiTextButton;
 	class Text;
 	class Selector;
 	class Number;
@@ -60,9 +61,12 @@ public:
 		GAMESHORTCUTS=13,
 		EDITORSHORTCUTS=14,
 		SECONDKEY=15,
+		PRESSEDSELECTOR=15,
 		ADDSHORTCUT=16,
 		REMOVESHORTCUT=17,
 		SCROLLWHEEL=18,
+		BUILDINGSETTINGS=19,
+		FLAGSETTINGS=20,
 	};
 private:
 	Settings old_settings;
@@ -71,14 +75,17 @@ private:
 	TextInput *userName;
 	
 	TextButton *ok, *cancel, *generalsettings, *unitsettings, *keyboardsettings;
+	TextButton *buildings, *flags;
 	OnOffButton *fullscreen, *usegpu, *lowquality, *customcur, *scrollwheel;
 	Selector *musicVol;
+	Selector *voiceVol;
 	OnOffButton *audioMute, *rememberUnitButton;
 	Number* unitRatios[IntBuildingType::NB_BUILDING][6];
 	Text* unitRatioTexts[IntBuildingType::NB_BUILDING][6];
+	int unitRatioGroupNumbers[IntBuildingType::NB_BUILDING][6];
 //	Text *title;
 	Text *language, *display, *usernameText, *audio;
-	Text *fullscreenText, *usegpuText, *lowqualityText, *customcurText, *musicVolText, *audioMuteText, *rememberUnitText, *scrollwheelText;
+	Text *fullscreenText, *usegpuText, *lowqualityText, *customcurText, *musicVolText, *audioMuteText, *voiceVolText, *rememberUnitText, *scrollwheelText;
 	Text *actDisplay;
 	Text *rebootWarning;
 
@@ -92,10 +99,15 @@ private:
 	KeySelector* select_key_1;
 	OnOffButton *key_2_active;
 	KeySelector* select_key_2;
+	MultiTextButton* pressedUnpressedSelector;
 	List* action_list;
 	TextButton* add_shortcut;
 	TextButton* remove_shortcut;
-
+	
+	Text* unitSettingsExplanation;
+	Text* unitSettingsHeading1;
+	Text* unitSettingsHeading2;
+	
 	bool gfxAltered;
 	
 	//! If GL is enabled, hide useless options
@@ -114,6 +126,15 @@ private:
 	KeyboardManager guiKeyboardManager;
 public:
 	ShortcutMode currentMode;
+	///Quick code that adds in a default unit assignment widget pair at the specific position, and returns the width.
+	int addDefaultUnitAssignmentWidget(int type, int level, int x, int y, int group, bool flag=false);
+	///Activates the given group number for default assignment widgets
+	void activateDefaultAssignedGroupNumber(int group);
+	///Returns the default unit assignment text
+	std::string getDefaultUnitAssignmentText(int type, int level, bool flag);
+	///Sets the texts for all default unit assignment widgets
+	void setLanguageTextsForDefaultAssignmentWidgets();
+	
 	///Update shortcut_list, if n is not -1, just update that specific entry
 	void updateShortcutList(int n=-1);
 	///Update the action_list
