@@ -217,7 +217,7 @@ public:
 	bool validTarget;
 	Sint32 magicActionTimeout;
 
-	// Timer counts down 5 frames after last attacked
+	// Timer counts down after last attacked
 	Uint8 underAttackTimer;
 
 	// trigger parameters
@@ -253,6 +253,11 @@ public:
 	int levelUpAnimation;
 	int magicActionAnimation;
 	
+	// These store the previous clearing area target cordinates
+	Uint32 previousClearingAreaX;
+	Uint32 previousClearingAreaY;
+	Uint32 previousClearingAreaDistance;
+	
 public:
 	// optimisation cached values
 	int stepsLeftUntilHungry;
